@@ -6,7 +6,7 @@
  * Descr.: all solution collector management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: all_solut_c.c,v 1.7 2002/09/12 16:35:28 diaz Exp $ */
+/* $Id: all_solut_c.c,v 1.12 2007/01/04 10:35:06 diaz Exp $ */
 
 #include <sys/types.h>
 
@@ -31,7 +31,7 @@
 #include "engine_pl.h"
 #include "bips_pl.h"
 
-#ifndef M_ix86_win32
+#ifndef _WIN32
 #include <unistd.h>
 #include <sys/wait.h>
 #endif
