@@ -6,7 +6,7 @@
  * Descr.: byte-code support                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.c,v 1.8 2002/05/07 17:45:45 diaz Exp $ */
+/* $Id: bc_supp.c,v 1.13 2007/01/04 10:35:07 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -570,7 +570,7 @@ BC_Emit_Inst_1(WamWord inst_word)
     case GET_STRUCTURE:
     case PUT_STRUCTURE:
       nb_word = 2;
-      BC_Fun(w) = BC_Arg_Func_Arity(*arg_adr++, &w1);
+      BC_Fun(w) = BC_Arg_Func_Arity(*arg_adr++, (int *) &w1);
       BC_X0(w) = Rd_Integer(*arg_adr);
       break;
 
@@ -602,7 +602,7 @@ BC_Emit_Inst_1(WamWord inst_word)
       break;
 
     case UNIFY_STRUCTURE:
-      BC_Fun(w) = BC_Arg_Func_Arity(*arg_adr++, &w1);
+      BC_Fun(w) = BC_Arg_Func_Arity(*arg_adr++, (int *) &w1);
       BC_Ari(w) = w1;
       break;
 
