@@ -6,7 +6,7 @@
  * Descr.: C interface support                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: c_supp.c,v 1.5 2002/03/19 18:24:34 diaz Exp $ */
+/* $Id: c_supp.c,v 1.11 2007/01/04 10:35:07 diaz Exp $ */
 
 #include <string.h>
 
@@ -2076,6 +2076,19 @@ Un_Callable(int func, int arity, WamWord *arg, WamWord start_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * UN_TERM                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Un_Term(WamWord term_word, WamWord start_word)
+{				/* used because Unify is FC convention */
+  return Unify(term_word, start_word);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * MK_INTEGER                                                              *
  *                                                                         *
