@@ -6,7 +6,7 @@
  * Descr.: meta call management                                            * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2002 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2007 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -19,10 +19,10 @@
  *                                                                         * 
  * You should have received a copy of the GNU General Public License along * 
  * with this program; if not, write to the Free Software Foundation, Inc.  * 
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: call.pl,v 1.7 2002/03/19 18:24:34 diaz Exp $ */
+/* $Id: call.pl,v 1.12 2007/01/04 10:35:08 diaz Exp $ */
 
 :-	built_in.
 
@@ -43,7 +43,7 @@ once(Goal) :-
 
 
 
-call(Goal, Deterministic) :-
+call_det(Goal, Deterministic) :-
 	(   nonvar(Deterministic),
 	    Deterministic \== false,
 	    Deterministic \== true ->
