@@ -6,7 +6,7 @@
  * Descr.: meta call management                                            * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2002 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2007 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -19,10 +19,10 @@
  *                                                                         * 
  * You should have received a copy of the GNU General Public License along * 
  * with this program; if not, write to the Free Software Foundation, Inc.  * 
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: call_args.pl,v 1.6 2002/03/19 18:24:34 diaz Exp $ */
+/* $Id: call_args.pl,v 1.11 2007/01/04 10:35:08 diaz Exp $ */
 
 :-	built_in.
 
@@ -63,3 +63,38 @@ call_with_args(_F, _A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8, _A9) :-
 
 call_with_args(_F, _A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8, _A9, _A10) :-
 	'$call_c_jump'('Call_With_Args_11').
+
+
+
+call(_Closure, _A1) :-
+	'$call_c_jump'('Call_2').
+
+call(_Closure, _A1, _A2) :-
+	'$call_c_jump'('Call_3').
+
+call(_Closure, _A1, _A2, _A3) :-
+	'$call_c_jump'('Call_4').
+
+call(_Closure, _A1, _A2, _A3, _A4) :-
+	'$call_c_jump'('Call_5').
+
+call(_Closure, _A1, _A2, _A3, _A4) :-
+	'$call_c_jump'('Call_5').
+
+call(_Closure, _A1, _A2, _A3, _A4, _A5) :-
+	'$call_c_jump'('Call_6').
+
+call(_Closure, _A1, _A2, _A3, _A4, _A5, _A6) :-
+	'$call_c_jump'('Call_7').
+
+call(_Closure, _A1, _A2, _A3, _A4, _A5, _A6, _A7) :-
+	'$call_c_jump'('Call_8').
+
+call(_Closure, _A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8) :-
+	'$call_c_jump'('Call_9').
+
+call(_Closure, _A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8, _A9) :-
+	'$call_c_jump'('Call_10').
+
+call(_Closure, _A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8, _A9, _A10) :-
+	'$call_c_jump'('Call_11').
