@@ -6,7 +6,7 @@
  * Descr.: file consulting - C part                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: consult_c.c,v 1.9 2002/04/19 15:43:35 diaz Exp $ */
+/* $Id: consult_c.c,v 1.14 2007/01/04 10:35:08 diaz Exp $ */
 
 #include <sys/types.h>
 
@@ -116,7 +116,7 @@ Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
 	      if (c == EOF)
 		{
 		eof_reached:
-		  p = "end_of_file.\n";
+		  p = (unsigned char *) "end_of_file.\n";
 		  c = *p++;
 		}
 	    }
