@@ -6,7 +6,7 @@
  * Descr.: Prolog errors support - header file                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: error_supp.h,v 1.7 2002/04/19 15:43:35 diaz Exp $ */
+/* $Id: error_supp.h,v 1.13 2007/01/04 10:35:09 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -251,7 +251,7 @@ extern int resource_too_big_fd_constraint; /* for FD */
 
 void Set_Bip_Name_2(WamWord func_word, WamWord arity_word);
 
-void Set_C_Bip_Name(char *func_str, int arity) FC;
+void FC Set_C_Bip_Name(char *func_str, int arity);
 
 void Unset_C_Bip_Name(void);
 
