@@ -6,7 +6,7 @@
  * Descr.: file name management - C part                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,17 +19,17 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: file_c.c,v 1.5 2002/04/10 13:10:43 diaz Exp $ */
+/* $Id: file_c.c,v 1.10 2007/01/04 10:35:09 diaz Exp $ */
 
 #include <string.h>
 
 #include "engine_pl.h"
 #include "bips_pl.h"
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
 #include <io.h>
 #else
 #include <unistd.h>
@@ -101,7 +101,7 @@ Decompose_File_Name_4(WamWord path_word, WamWord dir_word,
   Check_For_Un_Atom(suffix_word);
 
   p = strrchr(path, DIR_SEP_C);
-#ifdef M_ix86_win32
+#ifdef _WIN32
   {
     char *q = strrchr(path, '/');
 
@@ -178,7 +178,7 @@ Prolog_File_Name_2(WamWord f1_word, WamWord f2_word)
     }
 
   p = strrchr(f1, DIR_SEP_C);
-#ifdef M_ix86_win32
+#ifdef _WIN32
   {
     char *q = strrchr(f1, '/');
 
