@@ -6,7 +6,7 @@
  * Descr.: Prolog flag and system variable management - C Part             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_c.c,v 1.11 2002/04/09 08:26:40 diaz Exp $ */
+/* $Id: flag_c.c,v 1.16 2007/01/04 10:35:10 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,7 +35,7 @@
 #include "engine_pl.h"
 #include "bips_pl.h"
 
-#ifndef M_ix86_win32
+#ifndef _WIN32
 #include <unistd.h>
 extern char **environ;
 #endif
@@ -77,7 +77,7 @@ static int atom_toward_zero;
 static int atom_on;
 static int atom_off;
 
-static int atom_error;
+/* atom_error is already defined in the set of often used atoms */
 static int atom_warning;
 static int atom_fail;
 
@@ -150,7 +150,6 @@ Flag_Initializer(void)
   atom_on = Create_Atom("on");
   atom_off = Create_Atom("off");
 
-  atom_error = Create_Atom("error");
   atom_warning = Create_Atom("warning");
   atom_fail = Create_Atom("fail");
 
