@@ -6,7 +6,7 @@
  * Descr.: global variable (inline) management - C part                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: g_var_inl_c.c,v 1.7 2002/04/19 15:43:35 diaz Exp $ */
+/* $Id: g_var_inl_c.c,v 1.14 2007/01/04 10:35:10 diaz Exp $ */
 
 #include <stdlib.h>
 
@@ -45,7 +45,7 @@
 #define G_ARRAY_AUTO               1
 #define G_ARRAY_EXTEND             2
 
-#define G_IMPOSSIBLE_SIZE          ((unsigned long) -1 >> 1)
+#define G_IMPOSSIBLE_SIZE          ((unsigned int) -1 >> 1)
 #define MAX_AUTO_SIZE              (1 << 20)
 
 
@@ -179,7 +179,7 @@ G_Var_Initializer(void)
  * BLT_G_ASSIGN                                                            *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Assign(WamWord x, WamWord y)
 {
   Set_C_Bip_Name("g_assign", 2);
@@ -194,7 +194,7 @@ Blt_G_Assign(WamWord x, WamWord y)
  * BLT_G_ASSIGNB                                                           *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Assignb(WamWord x, WamWord y)
 {
   Set_C_Bip_Name("g_assignb", 2);
@@ -209,7 +209,7 @@ Blt_G_Assignb(WamWord x, WamWord y)
  * BLT_G_LINK                                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Link(WamWord x, WamWord y)
 {
   Set_C_Bip_Name("g_link", 2);
@@ -224,7 +224,7 @@ Blt_G_Link(WamWord x, WamWord y)
  * BLT_G_READ                                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Read(WamWord x, WamWord y)
 {
   Bool res;
@@ -242,7 +242,7 @@ Blt_G_Read(WamWord x, WamWord y)
  * BLT_G_ARRAY_SIZE                                                        *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Array_Size(WamWord x, WamWord y)
 {
   Bool res;
@@ -260,7 +260,7 @@ Blt_G_Array_Size(WamWord x, WamWord y)
  * BLT_G_INC                                                               *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Inc(WamWord x)
 {
   Set_C_Bip_Name("g_inc", 1);
@@ -275,7 +275,7 @@ Blt_G_Inc(WamWord x)
  * BLT_G_INCO                                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Inco(WamWord x, WamWord y)
 {
   Bool res;
@@ -293,7 +293,7 @@ Blt_G_Inco(WamWord x, WamWord y)
  * BLT_G_INC_2                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Inc_2(WamWord x, WamWord y)
 {
   Bool res;
@@ -311,7 +311,7 @@ Blt_G_Inc_2(WamWord x, WamWord y)
  * BLT_G_INC_3                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Inc_3(WamWord x, WamWord y, WamWord z)
 {
   Bool res;
@@ -329,7 +329,7 @@ Blt_G_Inc_3(WamWord x, WamWord y, WamWord z)
  * BLT_G_DEC                                                               *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Dec(WamWord x)
 {
   Set_C_Bip_Name("g_dec", 1);
@@ -344,7 +344,7 @@ Blt_G_Dec(WamWord x)
  * BLT_G_DECO                                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Deco(WamWord x, WamWord y)
 {
   Bool res;
@@ -362,7 +362,7 @@ Blt_G_Deco(WamWord x, WamWord y)
  * BLT_G_DEC_2                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Dec_2(WamWord x, WamWord y)
 {
   Bool res;
@@ -380,7 +380,7 @@ Blt_G_Dec_2(WamWord x, WamWord y)
  * BLT_G_DEC_3                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Dec_3(WamWord x, WamWord y, WamWord z)
 {
   Bool res;
@@ -398,7 +398,7 @@ Blt_G_Dec_3(WamWord x, WamWord y, WamWord z)
  * BLT_G_SET_BIT                                                           *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Set_Bit(WamWord x, WamWord y)
 {
   Set_C_Bip_Name("g_set_bit", 2);
@@ -413,7 +413,7 @@ Blt_G_Set_Bit(WamWord x, WamWord y)
  * BLT_G_RESET_BIT                                                         *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Blt_G_Reset_Bit(WamWord x, WamWord y)
 {
   Set_C_Bip_Name("g_reset_bit", 2);
@@ -428,7 +428,7 @@ Blt_G_Reset_Bit(WamWord x, WamWord y)
  * BLT_G_TEST_SET_BIT                                                      *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Test_Set_Bit(WamWord x, WamWord y)
 {
   Bool res;
@@ -446,7 +446,7 @@ Blt_G_Test_Set_Bit(WamWord x, WamWord y)
  * BLT_G_TEST_RESET_BIT                                                    *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_G_Test_Reset_Bit(WamWord x, WamWord y)
 {
   Bool res;
@@ -1303,7 +1303,7 @@ G_Array_Size(WamWord gvar_word, WamWord size_word)
 
   size = g_elem->size;
 
-  return g_arg != NULL && size < 0 && Get_Integer(-size, size_word);
+  return g_arg == NULL && size < 0 && Get_Integer(-size, size_word);
 }
 
 
