@@ -6,7 +6,7 @@
  * Descr.: operating system interface management - C part                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: os_interf_c.c,v 1.12 2002/09/27 14:20:58 diaz Exp $ */
+/* $Id: os_interf_c.c,v 1.19 2007/01/04 10:35:11 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,7 +36,7 @@
 
 #include "gp_config.h"
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
 #include <process.h>
 #include <direct.h>
 #include <io.h>
@@ -94,8 +94,7 @@ InfSig;
 
 static int atom_dt;
 
-static int atom_read;
-static int atom_write;
+/* atom_write is already defined in the set of often used atoms */
 static int atom_execute;
 static int atom_search;
 
@@ -140,8 +139,6 @@ Os_Interf_Initializer(void)
 {
   atom_dt = Create_Atom("dt");
 
-  atom_read = Create_Atom("read");
-  atom_write = Create_Atom("write");
   atom_execute = Create_Atom("execute");
   atom_search = Create_Atom("search");
 
@@ -217,7 +214,7 @@ Os_Interf_Initializer(void)
   tsig[nb_sig].atom = Create_Atom("SIGIO");
   tsig[nb_sig++].sig = SIGIO;
 #endif
-#if !defined( M_ix86_bsd ) && !defined( M_powerpc_darwin )
+#if !defined(M_bsd) && !defined(M_darwin)
   tsig[nb_sig].atom = Create_Atom("SIGPOLL");
   tsig[nb_sig++].sig = SIGPOLL;
 #endif
@@ -243,7 +240,7 @@ Make_Directory_1(WamWord path_name_word)
 
   path_name = Get_Path_Name(path_name_word);
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   Os_Test_Error(_mkdir(path_name));
 #else
   Os_Test_Error(mkdir(path_name, 0777));
@@ -321,7 +318,7 @@ Directory_Files_2(WamWord path_name_word, WamWord list_word)
   Bool res;
   char *name;
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   long h;
   struct _finddata_t d;
   static char buff[MAXPATHLEN];
@@ -335,7 +332,7 @@ Directory_Files_2(WamWord path_name_word, WamWord list_word)
 
   path_name = Get_Path_Name(path_name_word);
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   sprintf(buff, "%s\\*.*", path_name);
   h = _findfirst(buff, &d);	/* instead of Win32 FindFirstFile since uses errno */
   Os_Test_Error(h == -1);
@@ -344,7 +341,7 @@ Directory_Files_2(WamWord path_name_word, WamWord list_word)
   Os_Test_Error(dir == NULL);
 #endif
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   do
     {
       name = d.name;
@@ -361,14 +358,14 @@ Directory_Files_2(WamWord path_name_word, WamWord list_word)
 
       list_word = Unify_Variable();
     }
-#ifdef M_ix86_win32
+#ifdef _WIN32
   while (_findnext(h, &d) == 0);
 #endif
 
   res = Get_Nil(list_word);
 
 finish:
-#ifdef M_ix86_win32
+#ifdef _WIN32
   _findclose(h);
 #else
   closedir(dir);
@@ -581,7 +578,7 @@ File_Prop_Real_File_Name_2(WamWord real_path_name_word,
 {
   char *path_name = Get_Path_Name(path_name_word);
 
-#ifndef M_ix86_win32
+#ifndef _WIN32
   char real_path_name[MAXPATHLEN];
 
   Os_Test_Error(realpath(path_name, real_path_name) == NULL);
@@ -851,7 +848,7 @@ Architecture_1(WamWord architecture_word)
 void
 Sleep_1(WamWord seconds_word)
 {
-#ifdef M_ix86_win32
+#ifdef _WIN32
   DWORD ms;
 
   ms = (DWORD) (Rd_Number_Check(seconds_word) * 1000);
@@ -912,7 +909,7 @@ System_2(WamWord cmd_word, WamWord status_word)
   cmd = Rd_String_Check(cmd_word);
   Check_For_Un_Integer(status_word);
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   _flushall();
 #endif
 
@@ -1105,7 +1102,7 @@ Create_Pipe_2(WamWord stm_in_word, WamWord stm_out_word)
   FILE *f_in, *f_out;
   int atom;
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   Os_Test_Error(_pipe(p, 4096, O_TEXT));
 #else
   Os_Test_Error(pipe(p));
@@ -1138,7 +1135,7 @@ Bool
 Fork_Prolog_1(WamWord pid_word)
 
 {
-#if defined(M_ix86_win32)
+#ifdef _WIN32
 
   Pl_Err_Resource(Create_Atom("not implemented"));
   return FALSE;
@@ -1167,7 +1164,7 @@ Select_5(WamWord reads_word, WamWord ready_reads_word,
 	 WamWord writes_word, WamWord ready_writes_word,
 	 WamWord time_out_word)
 {
-#if defined(M_ix86_win32) && defined(NO_USE_SOCKETS)
+#if defined(_WIN32) && defined(NO_USE_SOCKETS)
 
   Pl_Err_Resource(Create_Atom("not implemented"));
   return FALSE;
@@ -1367,7 +1364,7 @@ Send_Signal_2(WamWord pid_word, WamWord signal_word)
   else
     sig = Rd_Integer_Check(word);
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   {
     int ret;
 
@@ -1407,7 +1404,7 @@ Wait_2(WamWord pid_word, WamWord status_word)
   pid = Rd_Integer_Check(pid_word);
   Check_For_Un_Integer(status_word);
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   Os_Test_Error(_cwait(&status, pid, _WAIT_CHILD) == -1);
 #else
   Os_Test_Error(waitpid(pid, &status, 0) == -1);
