@@ -6,7 +6,7 @@
  * Descr.: parser support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: parse_supp.c,v 1.8 2002/04/19 15:43:35 diaz Exp $ */
+/* $Id: parse_supp.c,v 1.13 2007/01/04 10:34:27 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -237,7 +237,7 @@ finish:
  * Recursively parses a term whose precedence is <=cur_prec and returns a  *
  * WamWord associated to the term. If a syntax error is encountered the    *
  * parser returns NOT_A_WAM_WORD and update syntax error information       *
- * (see Set_Syntax_Error_Info() in error_supp.c).                          *
+ * (see Set_Last_Syntax_Error_Info() in error_supp.c).                     *
  * The flag comma_is_punct specifies if an eventual ',' following the term *
  * must be considered as a punctuation (separator of args of compound term *
  * or of a list) or as an atom. The value COMMA_ANY is used when this flag *
@@ -367,8 +367,7 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
 	    }
 	  /* prefix operator as a name */
 	  if (context != GENERAL_TERM)
-	    Parse_Error
-	      ("expression expected or previous operator needs brackets");
+	    Parse_Error("expression expected or previous operator needs brackets");
 	}
 
       left_is_op = (Check_Oper_Any_Type(atom));
