@@ -6,7 +6,7 @@
  * Descr.: Prolog source file reader - C part                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: src_rdr_c.c,v 1.3 2002/05/07 17:45:46 diaz Exp $ */
+/* $Id: src_rdr_c.c,v 1.8 2007/01/04 10:35:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <errno.h>
@@ -30,7 +30,7 @@
 
 #include "gp_config.h"
 
-#ifndef M_ix86_win32
+#ifndef _WIN32
 #include <unistd.h>
 #else
 #include <io.h>
