@@ -6,7 +6,7 @@
  * Descr.: stream selection and control management - C part                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_c.c,v 1.11 2002/06/12 11:53:14 diaz Exp $ */
+/* $Id: stream_c.c,v 1.16 2007/01/04 10:35:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,11 +37,11 @@
 #include "linedit.h"
 #endif
 
-#if defined(M_ix86_cygwin) || defined(M_ix86_win32)
+#if defined(_WIN32) || defined(__CYGWIN__)
 #include <io.h>
 #endif
 
-#ifndef M_ix86_win32
+#ifndef _WIN32
 #include <unistd.h>
 #include <sys/fcntl.h>
 #endif
@@ -272,7 +272,7 @@ Open_3(WamWord source_sink_word, WamWord mode_word, WamWord stm_word)
 
 
   if ((mask & 4) != 0)		/* buffering specified */
-    if (prop.buffering != (mask & 3))
+    if (prop.buffering != (unsigned) (mask & 3)) /* cast for MSVC warning */
       {
 	prop.buffering = mask & 3;
 	Stdio_Set_Buffering(f, prop.buffering);
@@ -409,7 +409,7 @@ Set_Stream_Type_2(WamWord sora_word, WamWord is_text_word)
 		      permission_type_stream, sora_word);
 
   pstm->prop.text = text;
-#if defined(M_ix86_cygwin) || defined(M_ix86_win32)
+#if defined(_WIN32) || defined(__CYGWIN__)
   {
     FILE *f;
 
