@@ -6,7 +6,7 @@
  * Descr.: stream support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_supp.c,v 1.14 2002/09/19 11:00:35 diaz Exp $ */
+/* $Id: stream_supp.c,v 1.20 2007/01/04 10:35:12 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -51,7 +51,7 @@ extern void clearerr(FILE *stream);
 #include "linedit.h"
 #endif
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
 #include <io.h>
 #else
 #include <unistd.h>
@@ -237,7 +237,7 @@ Init_Stream_Supp(void)
   stm_stdout = Add_Stream_For_Stdio_Desc(stdout, atom_user_output,
 					 STREAM_MODE_WRITE, TRUE);
 
-#if !defined(NO_USE_LINEDIT) && defined(M_ix86_win32)
+#if !defined(NO_USE_LINEDIT) && defined(_WIN32)
 		/* ok for both GUI and console EOM<->ANSI conversion */
   pstm = stm_tbl[stm_stdout];
   pstm->prop.buffering = STREAM_BUFFERING_LINE;
@@ -926,7 +926,7 @@ Stdio_Set_Buffering(FILE *f, int buffering)
 
     case STREAM_BUFFERING_LINE:
       buff_flag = _IOLBF;
-#ifdef M_ix86_win32
+#ifdef _WIN32
       if (!use_gui)		/* in Win32 console app, line buff = full */
 	buff_flag = _IONBF;	/* I prefer no buffering */
 #endif
@@ -1066,7 +1066,7 @@ TTY_Getc(void)
 
   return c;
 }
-
+ 
 
 
 
