@@ -6,7 +6,7 @@
  * Descr.: test - C part                                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,7 +19,7 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
 /* $Id: t_c.c,v 1.14 2002/05/07 17:45:47 diaz Exp $ */
@@ -126,3 +126,43 @@ Bool malloc_test(int size)
 }
 #endif
 
+
+
+#include <string.h>
+#include "gprolog.h" 
+Bool
+calling_c(PlTerm In,PlTerm* Out)
+{
+  PlTerm arg[2];
+  int i;
+  
+  for(i=0;i<2;i++)
+    arg[i]=X(1);
+
+  Pl_Query_Begin(TRUE);
+  Pl_Query_Call(Find_Atom("write"),1,&In);
+  Pl_Query_End(PL_RECOVER);
+
+
+  for(i=0;i<2;i++)
+    X(1)=arg[i];
+
+  *Out = Mk_Atom(atom_nil);
+
+  return PL_SUCCESS; 
+} 
+
+
+
+
+Bool 
+Is_Kbd_Empty(void)
+{
+  /* to put in stream_supp.c
+  if (tty_ptr != NULL && *tty_ptr != '\0')
+    return FALSE;
+  */
+  int result = !LE_Kbd_Is_Not_Empty();
+  printf("result: %d\n", result);
+  return result;
+}
