@@ -6,7 +6,7 @@
  * Descr.: term support                                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_supp.c,v 1.8 2002/10/01 12:56:21 diaz Exp $ */
+/* $Id: term_supp.c,v 1.14 2007/01/04 10:35:12 diaz Exp $ */
 
 #include <string.h>
 
@@ -126,7 +126,7 @@ Term_Compare(WamWord start_u_word, WamWord start_v_word)
 	  v_tag == FLT)
 	return 1;
 
-      return (v_tag != INT) ? -1 : u_word - v_word;
+      return (v_tag != INT) ? -1 : UnTag_INT(u_word) - UnTag_INT(v_word);
 
     case ATM:
       if (v_tag == REF ||
