@@ -6,7 +6,7 @@
  * Descr.: top Level                                                       * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2002 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2007 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -19,10 +19,10 @@
  *                                                                         * 
  * You should have received a copy of the GNU General Public License along * 
  * with this program; if not, write to the Free Software Foundation, Inc.  * 
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_level.pl,v 1.9 2002/04/19 15:43:36 diaz Exp $ */
+/* $Id: top_level.pl,v 1.15 2007/01/04 10:35:12 diaz Exp $ */
 
 :-	built_in.
 
@@ -34,7 +34,7 @@ top_level :-
 	current_prolog_flag(prolog_version, Version),
 	current_prolog_flag(prolog_copyright, Copyright),
 	format(top_level_output, '~N~a ~a~n', [Name, Version]),
-	write('By Daniel Diaz'),
+	write(top_level_output, 'By Daniel Diaz'),
 	nl,
 	format(top_level_output, '~a~n', [Copyright]),
 	break.
@@ -211,12 +211,12 @@ break :-
 	'$call'(X, top_level, 0, true),
 	'$get_current_B'(B1),
 	format(top_level_output, '~N', []),
-	'$set_query_vars_names'(QueryVars, ToDispVars1),
+	'$set_query_vars_names'(QueryVars, ToDispVars),
 	(   fail,                             % do not activate 'alt if vars'
-	    ToDispVars1 = [] ->
+	    ToDispVars = [] ->
 	    true                              % no alt if only anonymous vars
 	;
-	    '$write_solution'(ToDispVars1, B1, B),
+	    '$write_solution'(ToDispVars, B1, B),
 	    (   B1 > B ->
 	        g_read('$all_solutions', f),          % fail for previous 'a'
 	        write(top_level_output, ' ? '),
@@ -228,7 +228,7 @@ break :-
 
 
 
-'$set_query_vars_names'(QueryVars, ToDispVars) :-
+'$set_query_vars_names'(QueryVars, ToDispVars1) :-
 	name_query_vars(QueryVars, ToDispVars),
 	'$remove_underscore_vars'(ToDispVars, ToDispVars1),
 	name_singleton_vars(ToDispVars1),
