@@ -6,7 +6,7 @@
  * Descr.: type testing (inline) management - C part                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: type_inl_c.c,v 1.6 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: type_inl_c.c,v 1.12 2007/01/04 10:35:12 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -87,79 +87,79 @@
 
 
 
-Bool
+Bool FC
 Blt_Var(WamWord x)
 {
   Type_Test(Tag_Is_Var, x);
 }
 
-Bool
+Bool FC
 Blt_Non_Var(WamWord x)
 {
   Type_Test(Tag_Is_Nonvar, x);
 }
 
-Bool
+Bool FC
 Blt_Atom(WamWord x)
 {
   Type_Test(Tag_Is_Atom, x);
 }
 
-Bool
+Bool FC
 Blt_Integer(WamWord x)
 {
   Type_Test(Tag_Is_Integer, x);
 }
 
-Bool
+Bool FC
 Blt_Float(WamWord x)
 {
   Type_Test(Tag_Is_Float, x);
 }
 
-Bool
+Bool FC
 Blt_Number(WamWord x)
 {
   Type_Test(Tag_Is_Number, x);
 }
 
-Bool
+Bool FC
 Blt_Atomic(WamWord x)
 {
   Type_Test(Tag_Is_Atomic, x);
 }
 
-Bool
+Bool FC
 Blt_Compound(WamWord x)
 {
   Type_Test(Tag_Is_Compound, x);
 }
 
-Bool
+Bool FC
 Blt_Callable(WamWord x)
 {
   Type_Test(Tag_Is_Callable, x);
 }
 
-Bool
+Bool FC
 Blt_Fd_Var(WamWord x)
 {
   Type_Test(Tag_Is_Fd_Var, x);
 }
 
-Bool
+Bool FC
 Blt_Non_Fd_Var(WamWord x)
 {
   Type_Test(Tag_Is_Non_Fd_Var, x);
 }
 
-Bool
+Bool FC
 Blt_Generic_Var(WamWord x)
 {
   Type_Test(Tag_Is_Generic_Var, x);
 }
 
-Bool
+Bool FC
 Blt_Non_Generic_Var(WamWord x)
 {
   Type_Test(Tag_Is_Non_Generic_Var, x);
@@ -172,7 +172,7 @@ Blt_Non_Generic_Var(WamWord x)
  * BLT_LIST                                                                *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_List(WamWord start_word)
 {
   WamWord word, tag_mask;
@@ -198,7 +198,7 @@ Blt_List(WamWord start_word)
  * BLT_PARTIAL_LIST                                                        *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_Partial_List(WamWord start_word)
 {
   WamWord word, tag_mask;
@@ -224,7 +224,7 @@ Blt_Partial_List(WamWord start_word)
  * BLT_LIST_OR_PARTIAL_LIST                                                *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 Blt_List_Or_Partial_List(WamWord start_word)
 {
   WamWord word, tag_mask;
