@@ -6,7 +6,7 @@
  * Descr.: FD instruction implementation                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_inst.c,v 1.9 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: fd_inst.c,v 1.14 2007/01/04 10:35:13 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -993,7 +993,7 @@ Fd_Tell_Range_Range(WamWord *fdv_adr, Range *range)
   WamWord *save_CS = CS;
 
   if (range->vec)
-    CS = range->vec;
+    CS = (WamWord *) range->vec;
   CS += vec_size;
 
   Range_Inter(range, Range(fdv_adr));
@@ -1096,7 +1096,7 @@ Fd_After_Add_Cstr(void)
 
 #if 1
 		/* optim #2 */
-		pdate = Optim_Pointer(CF);
+		pdate = (unsigned long *) Optim_Pointer(CF);
 		if (*pdate < date)
 		  continue;
 #endif
