@@ -6,7 +6,7 @@
  * Descr.: architecture dependent features - Header file                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,16 +19,30 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arch_dep.h,v 1.7 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: arch_dep.h,v 1.14 2007/01/04 10:35:13 diaz Exp $ */
 
 #ifndef _ARCH_DEP_H
 #define _ARCH_DEP_H
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 
+/* There are 2 kinds of MSVC warning C4996 one wants to remove:
+ * 1) XXX was declared deprecated ... This function or variable may be unsafe
+ *    solution: #define _CRT_SECURE_NO_DEPRECATE 1
+ * 2) The POSIX name for this item is deprecated
+ *    solution: #define _CRT_NONSTDC_NO_DEPRECATE 1
+ * However, these defines only work if they are before any #include <...>
+ * So: pass to cl: -D_CRT_SECURE_NO_DEPRECATE -D_CRT_NONSTDC_NO_DEPRECATE 
+ * or deactivate the warning with the following pragma. We do both !
+ */
+
+#ifdef _MSC_VER
+#pragma warning(disable : 4996)
+#endif
+
 #define MAXPATHLEN                 1024
 #define SIGQUIT                    SIGTERM
 #define fdopen                     _fdopen
@@ -46,19 +60,27 @@
 #define unlink                     _unlink
 #define tzset                      _tzset
 #define access                     _access
+
+#ifndef F_OK
 #define F_OK                       00
 #define W_OK                       02
 #define R_OK                       04
 #define X_OK                       F_OK
+#endif
+
 #ifndef S_ISDIR
 #define	S_ISDIR(m)	           (((m)&_S_IFMT) == _S_IFDIR)
 #define	S_ISCHR(m)                 (((m)&_S_IFMT) == _S_IFCHR)
 #define	S_ISFIFO(m)	           (((m)&_S_IFMT) == _S_IFIFO)
 #define	S_ISREG(m)	           (((m)&_S_IFMT) == _S_IFREG)
 #endif
+
+#ifndef S_IRUSR
 #define S_IRUSR                    _S_IREAD
 #define S_IWUSR                    _S_IWRITE
 #define S_IXUSR                    _S_IEXEC
+#endif
+
 #define DIR_SEP_S                  "\\"
 #define DIR_SEP_C                  '\\'
 
@@ -67,11 +89,11 @@
 #define DIR_SEP_S                  "/"
 #define DIR_SEP_C                  '/'
 
-#endif
+#endif /* defined(_WIN32) && !defined(__CYGWIN__) */
 
 #if defined(M_ix86_cygwin) || defined(M_ix86_sco)
 #define Set_Line_Buf(s)            setvbuf(s, NULL, _IOLBF, 0)
-#elif defined(M_ix86_win32)
+#elif defined(_WIN32)
 #define Set_Line_Buf(s)            setbuf(s, NULL)
 #else
 #define Set_Line_Buf(s)            setlinebuf(s)
@@ -116,19 +138,112 @@
 #define FC_MAX_ARGS_IN_REGS 3	
 #define FC_SET_OF_REGISTERS { "%eax", "%edx", "%ecx" };
 #define FC_ATTRIB __attribute__((regparm(FC_MAX_ARGS_IN_REGS)))
-#else  /* MSVC++ ? */
-/* TO DO */
+#elif 0  /* under MSVC++ we can use __fastcall convention (#elif 1 if wanted) */
+         /* see file ix86_any.c to see why it is not selected by default */
+#define FC_MAX_ARGS_IN_REGS 2
+#define FC_SET_OF_REGISTERS { "%ecx", "%edx" };
+#define FC_ATTRIB __fastcall
+#else
+#define FC_MAX_ARGS_IN_REGS 0
+#define FC_SET_OF_REGISTERS { NULL };
+#define FC_ATTRIB
 #endif
 
 #endif
 
-
 #if !defined(NO_USE_FAST_CALL) && defined(FC_ATTRIB)
 #define FC_USED_TO_COMPILE_CORE
+#ifndef FC /* to compile Ma2Asm/check.c without FC */
 #define FC FC_ATTRIB
+#endif
 #else
 #define FC
 #endif
 
 
+/* Win32 SEH macros */
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+				/* from MSVC++ windows.h + renaming */
+typedef enum {
+    ExceptContinueExecution,
+    ExceptContinueSearch,
+    ExceptNestedException,
+    ExceptCollidedUnwind
+} EXCEPT_DISPOSITION;
+
+typedef struct _excp_lst
+{
+  struct _excp_lst *chain;
+  EXCEPT_DISPOSITION (*handler)();
+} excp_lst;
+  
+
+#ifdef __GNUC__
+
+#  define SEH_PUSH(new_handler)			\
+{						\
+  excp_lst e;					\
+  EXCEPT_DISPOSITION new_handler();		\
+  e.handler = new_handler;			\
+  asm("movl %%fs:0,%0" : "=r" (e.chain));	\
+  asm("movl %0,%%fs:0" : : "r" (&e));
+
+ 
+#  define SEH_POP				\
+  asm("movl %0,%%fs:0" : : "r" (e.chain));	\
+}
+
+#elif defined(_MSC_VER)
+
+#  define SEH_PUSH(new_handler)			\
+{						\
+  excp_lst e;					\
+  EXCEPT_DISPOSITION new_handler();		\
+  e.handler = new_handler;			\
+  __asm push eax				\
+  __asm mov eax,dword ptr fs:[0]		\
+  __asm mov dword ptr [e.chain],eax		\
+  __asm lea eax,[e]				\
+  __asm mov dword ptr fs:[0],eax		\
+  __asm pop eax
+
+#  define SEH_POP				\
+  __asm push eax				\
+  __asm mov eax,dword ptr [e.chain]		\
+  __asm mov dword ptr fs:[0],eax		\
+  __asm pop eax					\
+}
+
+#elif defined(__LCC__)
+ /* below in movl %eax,%e and movel %e,%eax %e should be %e.chain the lcc asm 
+    does not support it. Here %e works since chain is the 1st field */
+#  define SEH_PUSH(new_handler)			\
+{						\
+  excp_lst e;					\
+  EXCEPT_DISPOSITION new_handler();		\
+  e.handler = new_handler;			\
+  _asm("pushl %eax");				\
+  _asm("movl %fs:0,%eax");			\
+  _asm("movl %eax,%e");				\
+  _asm("leal %e,%eax");				\
+  _asm("movl %eax,%fs:0");			\
+  _asm("popl %eax");
+
+#  define SEH_POP				\
+  _asm("pushl %eax");				\
+  _asm("movl %e,%eax");				\
+  _asm("movl %eax,%fs:0");			\
+  _asm("popl %eax");				\
+}
+
+#else
+
+#  error macros SEH_PUSH/POP undefined for this compiler
+
+#endif
+
+#endif /* defined(_WIN32) || defined(__CYGWIN__) */
+
 #endif /* !_ARCH_DEP_H */
+
