@@ -6,7 +6,7 @@
  * Descr.: atom table management                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom.c,v 1.6 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: atom.c,v 1.13 2007/01/04 10:35:13 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -120,7 +120,7 @@ static AtomInf *Locate_Atom(char *name);
 
 static int Hash_String(char *str);
 
-static char *Gen_Sym(unsigned char *prefix, int gen_sym_hash);
+static char *Gen_Sym(char *prefix, int gen_sym_hash);
 
 
 
@@ -225,7 +225,7 @@ Init_Atom(void)
  * CREATE_ALLOCATE_ATOM                                                    *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-int
+int FC
 Create_Allocate_Atom(char *name)
 {
   AtomInf *patom;
@@ -252,7 +252,7 @@ Create_Allocate_Atom(char *name)
  *                                                                         *
  * Called by compiled prolog code.                                         *
  *-------------------------------------------------------------------------*/
-int
+int FC
 Create_Atom(char *name)
 {
   AtomInf *patom;
@@ -263,7 +263,6 @@ Create_Atom(char *name)
   Bool identifier;
   Bool graphic;
 
-
   patom = Locate_Atom(name);
 
   if (patom == NULL)
@@ -346,7 +345,7 @@ finish:
  *                                                                         *
  * Called by compiled prolog code.                                         *
  *-------------------------------------------------------------------------*/
-WamWord
+WamWord FC
 Create_Atom_Tagged(char *name)
 {
   return Tag_ATM(Create_Atom(name));
@@ -360,7 +359,7 @@ Create_Atom_Tagged(char *name)
  *                                                                         *
  * return the atom key or -1 if not exist.                                 *
  *-------------------------------------------------------------------------*/
-int
+int FC
 Find_Atom(char *name)
 {
   AtomInf *patom;
@@ -450,7 +449,7 @@ Hash_String(char *str)
  * Find a new atom (gensym) beginning by a given prefix.                   *
  * hash<0 for any input or the index of the free atom to produce.          *
  *-------------------------------------------------------------------------*/
-int
+int FC
 Gen_New_Atom(char *prefix, int hash)
 {
   AtomInf *patom;
@@ -478,7 +477,7 @@ Gen_New_Atom(char *prefix, int hash)
  * returns a string beginning by a prefix st. its hash code is gen_sym_hash*
  *-------------------------------------------------------------------------*/
 static char *
-Gen_Sym(unsigned char *prefix, int gen_sym_hash)
+Gen_Sym(char *prefix, int gen_sym_hash)
 {
   unsigned pl = strlen(prefix);
   unsigned hp = Hash_String(prefix);
@@ -532,7 +531,7 @@ Gen_Sym(unsigned char *prefix, int gen_sym_hash)
  *                                                                         *
  * returns the atom next after 'last_atom' (-1 to start) or -1 at the end  *
  *-------------------------------------------------------------------------*/
-int
+int FC
 Find_Next_Atom(int last_atom)
 {
   while (++last_atom < MAX_ATOM)
