@@ -6,7 +6,7 @@
  * Descr.: atom table management - header file                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom.h,v 1.5 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: atom.h,v 1.11 2007/01/04 10:35:13 diaz Exp $ */
 
 #if 1
 #define OPTIM_1_CHAR_ATOM
@@ -165,17 +165,17 @@ extern char escape_char[];
 
 void Init_Atom(void);
 
-int Create_Allocate_Atom(char *name) FC;
+int FC Create_Allocate_Atom(char *name);
 
-int Create_Atom(char *name) FC;
+int FC Create_Atom(char *name);
 
-WamWord Create_Atom_Tagged(char *name) FC;
+WamWord FC Create_Atom_Tagged(char *name);
 
-int Find_Atom(char *name) FC;
+int FC Find_Atom(char *name);
 
-int Gen_New_Atom(char *prefix, int hash) FC;
+int FC Gen_New_Atom(char *prefix, int hash);
 
-int Find_Next_Atom(int last_atom) FC;
+int FC Find_Next_Atom(int last_atom);
 
 
 
