@@ -6,7 +6,7 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine.c,v 1.8 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: engine.c,v 1.16 2007/01/04 10:35:13 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,7 +31,7 @@
 #include <locale.h>
 
 #include "gp_config.h"
-#ifdef M_ix86_win32
+#if defined(_WIN32) || defined(__CYGWIN__)
 #include <windows.h>
 #endif
 
@@ -318,17 +318,13 @@ Call_Prolog(CodePtr codep)
   ALTB(query_b) = (CodePtr) Call_Prolog_Fail;	/* modify choice point */
 
   CP = Adjust_CP(Call_Prolog_Success);
-#ifdef M_ix86_win32
-  _try
-  {
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+  SEH_PUSH(Win32_SEH_Handler);
 #endif
-    ok = Call_Next(codep);
-#ifdef M_ix86_win32
-  }
-  _except(Is_Win32_SEGV(GetExceptionInformation()))
-  {
-    SIGSEGV_Handler();
-  }
+  ok = Call_Next(codep);
+#if defined(_WIN32) || defined(__CYGWIN__)
+  SEH_POP;
 #endif
 
   CP = save_CP;			/* restore continuation */
@@ -409,12 +405,16 @@ Call_Next(CodePtr codep)
   jmp_buf new_jumper;
   WamWord *old_buff_save = p_buff_save;
   WamWord buff_save_machine_regs[NB_OF_USED_MACHINE_REGS + 1];	/* +1 if = 0 */
+#if 0
   WamWord buff_save_all_regs[NB_OF_REGS];
+#endif
 
   p_jumper = &new_jumper;
   p_buff_save = buff_save_machine_regs;
 
+#if 0
   Save_All_Regs(buff_save_all_regs);
+#endif
   Save_Machine_Regs(buff_save_machine_regs);
 
   jmp_val = setjmp(*p_jumper);
@@ -427,13 +427,15 @@ Call_Next(CodePtr codep)
   if (jmp_val == 3)		/* return with a continuation in jmp_val */
     Call_Compiled(cont_jmp);
 
-  /* normal return */
+				/* normal return */
   p_jumper = old_jumper;
   p_buff_save = old_buff_save;
 
   if (jmp_val < 0)		/* false: restore WAM registers */
     {
+#if 0
       Restore_All_Regs(buff_save_all_regs);
+#endif
       return FALSE;
     }
 
@@ -451,6 +453,11 @@ Call_Next(CodePtr codep)
 static void
 Call_Prolog_Fail(void)
 {
+#ifdef M_ix86_darwin		/* see comment in Ma2Asm/ix86_any.c */
+  asm("subl $4,%esp");
+#elif defined(M_x86_64_linux) 	/* see comment in Ma2Asm/x86_64_any.c */
+  asm("subq $8,%rsp");
+#endif
   Save_Machine_Regs(p_buff_save);
   longjmp(*p_jumper, -1);
 }
@@ -466,6 +473,11 @@ Call_Prolog_Fail(void)
 static void
 Call_Prolog_Success(void)
 {
+#ifdef M_ix86_darwin		/* see comment in Ma2Asm/ix86_any.c */
+  asm("subl $4,%esp");
+#elif defined(M_x86_64_linux) 	/* see comment in Ma2Asm/x86_64_any.c */
+  asm("subq $8,%rsp");
+#endif
   Save_Machine_Regs(p_buff_save);
   longjmp(*p_jumper, 1);
 }
