@@ -6,7 +6,7 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine1.c,v 1.6 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: engine1.c,v 1.17 2007/01/04 10:35:13 diaz Exp $ */
 
 #include "engine_pl.h"
 
@@ -46,7 +46,7 @@ WamWord *ensure_reserved;
  * This functions is in a separate file for historical reasons. In versions*
  * <= 1.2.4, this file was compiled without any C compiler optimization to *
  * ensure reserved_stack_space was not removed but the C compiler. In order*
- * to use ebp under ix86 it muet be compiled with -fmoit-frame-pointer. The*
+ * to use ebp under ix86 it must be compiled with -fomit-frame-pointer. The*
  * simpliest way was to use the same C compiler invocation but adding a    *
  * global variable to ensure the stack is not removed.                     *
  *-------------------------------------------------------------------------*/
@@ -56,15 +56,22 @@ Call_Compiled(CodePtr codep)
 {
   WamWord reserved_stack_space[1024];
 
+#if defined(M_sparc) && !defined(M_sparc_bsd)
+  register long * __attribute__ ((unused)) rfl asm("%l2") = base_fl;
+  register double * __attribute__ ((unused)) rfd asm("%l3") = base_fd;
+  ensure_reserved = (WamWord *) rfl + (long) rfd; /* to avoid gcc remove 2 previous inits ! */
+#endif
+
 #if !defined(NO_MACHINE_REG_FOR_REG_BANK) && !defined(MAP_REG_BANK)
 
-#if defined(M_ix86_linux) || defined(M_ix86_cygwin) || \
-    defined(M_ix86_sco) || defined(M_ix86_bsd)
+#if defined(M_ix86_linux) || \
+    defined(M_ix86_cygwin) || defined(M_ix86_mingw) || \
+    defined(M_ix86_sco) || defined(M_ix86_bsd) 
 
   register WamWord *rb asm("%ebx") = reg_bank;
   ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
-#elif defined(M_ix86_win32)
+#elif defined(_MSC_VER)
 
   _asm mov ebx, reg_bank
 
@@ -93,15 +100,24 @@ Call_Compiled(CodePtr codep)
   register WamWord *rb asm("%l0") = reg_bank;
   ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
-#endif
+#elif defined(M_x86_64_linux)
+
+  register WamWord *rb asm("%r12") = reg_bank;
+  ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
 #endif
 
-#ifdef M_sparc
-  register long *rfl asm("%l2") = base_fl;
-  register double *rfd asm("%l3") = base_fd;
 #endif
 
   ensure_reserved = reserved_stack_space;
+#if defined(M_ix86_darwin)		/* see comment in Ma2Asm/ix86_any.c */
+  asm("andl $0xfffffff0,%esp");
+  asm("addl $4,%esp");
+#elif defined(M_x86_64_linux)		/* see comment in Ma2Asm/x86_64_any.c */
+  asm("andq $0xfffffffffffffff0,%rsp");
+  asm("addq $8,%rsp");
+#endif
+
   (*codep) ();
+
 }
