@@ -6,7 +6,7 @@
  * Descr.: FD interface for Prolog engine                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: if_no_fd.c,v 1.4 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: if_no_fd.c,v 1.9 2007/01/04 10:35:13 diaz Exp $ */
 
 #include <stdio.h>
 
@@ -40,7 +40,7 @@
 
 	  /* Errors Messages */
 
-#define ERR_FD_SOLVER_MISSING      "fd_interf.c: FD Solver not linked"
+#define ERR_FD_SOLVER_MISSING      __FILE__ ": FD Solver not linked"
 
 
 
