@@ -6,7 +6,7 @@
  * Descr.: machine dependent features - Header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,13 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.h,v 1.9 2002/05/10 16:35:25 diaz Exp $ */
+/* $Id: machine.h,v 1.18 2007/01/04 10:35:13 diaz Exp $ */
+
+#ifndef _MACHINE_H
+#define _MACHINE_H
 
 #include "bool.h"
 
@@ -78,16 +81,14 @@ Bool M_Set_Working_Dir(char *path);
 
 char *M_Absolute_Path_Name(char *src);
 
-#ifdef M_ix86_win32
+#if defined(_WIN32) && !defined(__CYGWIN__)
 
 int getpagesize(void);
 
-int Is_Win32_SEGV(void *exp);
-
-void SIGSEGV_Handler(void);
-
 #endif
 
+void M_Check_Magic_Words(void); /* not compiled if not needed */
+
 
 
 
@@ -95,29 +96,21 @@ void SIGSEGV_Handler(void);
  * Register Definitions            *
  *---------------------------------*/
 
-#if defined(M_sparc_sunos)
+#if defined(M_sparc)
 
 #    define M_USED_REGS            {"g6", "g7", 0}
 
-#elif defined(M_sparc_solaris)
-
-#    define M_USED_REGS            {"g6", "g7", 0}
-
-#elif defined(M_mips_irix)
+#elif defined(M_mips)
 
 #define M_USED_REGS                {"$16", "$17", "$18", "$19", "$20", \
                                     "$21", "$22", "$23", 0}
 
-#elif defined(M_alpha_linux)
+#elif defined(M_alpha)
 
 #    define M_USED_REGS            {"$9", "$10", "$11", "$12", "$13", "$14", 0}
 
-#elif defined(M_alpha_osf)
-
-#    define M_USED_REGS            {"$9", "$10", "$11", "$12", "$13", "$14", 0}
-
-#elif defined(M_ix86_linux)   || defined(M_ix86_sco) || \
-      defined(M_ix86_solaris) || defined(M_ix86_cygwin)
+/* on M_ix86_darwin : %ebx is used by gcc for pic base */
+#elif defined(M_ix86) && !defined(_MSC_VER) && !defined(M_ix86_darwin)
 
 #ifdef NO_USE_EBP
 #    define M_USED_REGS            {"ebx", 0}
@@ -125,13 +118,13 @@ void SIGSEGV_Handler(void);
 #    define M_USED_REGS            {"ebx", "ebp", 0}
 #endif
 
-#elif defined(M_powerpc_linux)
+#elif defined(M_powerpc)
 
 #    define M_USED_REGS            {"15", "20", 0}
 
-#elif defined(M_powerpc_darwin)
+#elif defined(M_x86_64)
 
-#    define M_USED_REGS            {"15", "20", 0}
+#    define M_USED_REGS            {"r12", "r13", "r14", "r15", 0}
 
 #else
 
@@ -140,7 +133,7 @@ void SIGSEGV_Handler(void);
 #endif
 
 
-#if defined(M_ix86) && !defined(M_ix86_win32) && !defined(NO_USE_REGS)
+#if defined(M_ix86) && !defined(_WIN32) && !defined(NO_USE_REGS)
 #define NO_MACHINE_REG_FOR_REG_BANK
 #endif
 
@@ -151,27 +144,19 @@ void SIGSEGV_Handler(void);
  * Stacks Management               *
  *---------------------------------*/
 
-#if defined(M_sparc_sunos) || defined(M_sparc_solaris)  || \
-    defined(M_ix86_linux)  || defined(M_powerpc_linux)  || \
-    defined(M_ix86_sco)    || defined(M_ix86_solaris)   || \
-    defined(M_mips_irix)   || defined(M_powerpc_darwin) || \
-    defined(M_ix86_win32)
+#if WORD_SIZE == 32
 
-#   define M_USE_MMAP
-#   define M_MMAP_HIGH_ADR         0x0ffffff0
-#   define M_MMAP_HIGH_ADR_ALT     0x3ffffff0
-#   define M_Check_Stacks()
+#   define M_MMAP_HIGH_ADR1        0x0ffffff0
+#   define M_MMAP_HIGH_ADR2        0x3ffffff0
+#   define M_MMAP_HIGH_ADR3        0x7ffffff0
 
 #elif defined(M_alpha_osf) || defined(M_alpha_linux)
 
-#   define M_USE_MMAP
-#   define M_MMAP_HIGH_ADR         0x3f800000000ULL
-#   define M_Check_Stacks()
+#   define M_MMAP_HIGH_ADR1        0x3f800000000ULL
 
-#else
+#elif defined(M_x86_64_linux)
 
-#   define M_USE_MALLOC
-#   define M_Check_Stacks()        M_Check_Magic_Words()
+#   define M_MMAP_HIGH_ADR1        0x4000000000ULL
 
 #endif
 
@@ -181,9 +166,13 @@ void SIGSEGV_Handler(void);
 
 
 
-#if defined(M_USE_MALLOC) || defined(M_powerpc_darwin)
+/*---------------------------------*
+ * Malloc Management               *
+ *---------------------------------*/
+
+#ifdef __OpenBSD__
+#define USE_DL_MALLOC
+#endif
 
-#define M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
-void M_Check_Magic_Words(void);
 
 #endif
