@@ -6,7 +6,7 @@
  * Descr.: malloc with checks + other miscellaneous operations             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,22 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: misc.c,v 1.7 2002/04/09 08:26:41 diaz Exp $ */
+/* $Id: misc.c,v 1.12 2007/01/04 10:35:13 diaz Exp $ */
+
+#include "machine.h"
+
+#ifdef USE_DL_MALLOC
+#include "dl_malloc.c"
+
+static void __attribute__((constructor)) 
+Init_Dl_Malloc(void) {
+  mallopt(M_MMAP_THRESHOLD, 0xFFFFFFF);	/* big value to no use mmap */
+}
+#endif
+
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -208,5 +220,7 @@ Fatal_Error(char *format, ...)
 #endif
     fprintf(stderr, "\nFatal Error: %s\n", buff);
 
-  exit(1);
+  Exit_With_Value(1);
 }
+
+
