@@ -6,7 +6,7 @@
  * Descr.: object chaining management - header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,43 +19,20 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: obj_chain.h,v 1.4 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: obj_chain.h,v 1.12 2007/01/04 10:35:13 diaz Exp $ */
+
 
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
 
-#define OBJ_CHAIN_MAGIC_1          0xdeadbeef
-#define OBJ_CHAIN_MAGIC_2          0x12345678
-
-
-
-
 /*---------------------------------*
  * Type Definitions                *
  *---------------------------------*/
 
-#ifndef _MSC_VER
-
-typedef struct objchain *PObjChain;
-
-typedef struct objchain
-{
-  long magic1;
-  long magic2;
-  PObjChain *next;
-  void (*fct_init) ();
-}
-ObjChain;
-
-#endif
-
-
-
-
 /*---------------------------------*
  * Global Variables                *
  *---------------------------------*/
@@ -66,31 +43,47 @@ ObjChain;
 
 void Find_Linked_Objects(void);
 
-void New_Object(void (*fct_exec_system) (), void (*fct_exec_user) ());
-
+void New_Object(void (*fct_obj_init)(), void (*fct_exec_system) (), void (*fct_exec_user) ());
 
 
 #ifdef OBJ_INIT
 
-static void (OBJ_INIT) ();
+static void OBJ_INIT(void);
 
-#ifndef _MSC_VER
+#define CPP_CAT1(x, y)   x ## y
+#define CPP_CAT(x, y)    CPP_CAT1(x, y)
 
-extern ObjChain *obj_chain_stop;
+#define OBJ_CTOR  CPP_CAT(OBJ_INIT,_ctor)
 
-static ObjChain obj_chain_start =
-  { OBJ_CHAIN_MAGIC_1, OBJ_CHAIN_MAGIC_2, &obj_chain_stop, OBJ_INIT };
 
-static ObjChain *obj_chain_stop = &obj_chain_start;
+#ifdef __GNUC__
+static void __attribute__ ((constructor))
+OBJ_CTOR(void)
+{
+  New_Object(OBJ_INIT, NULL, NULL);
+}
 
-#else
 
-#pragma data_seg(".INIT$b")
+#else /* _MSC_VER */
 
-static long obj_chain_start = (long) OBJ_INIT;
+static void 
+OBJ_CTOR(void)
+{
+  New_Object(OBJ_INIT, NULL, NULL);
+}
+
+#pragma data_seg(".GPLC$m")
+
+static long obj_chain_start = (long) OBJ_CTOR;
 
 #pragma data_seg()
 
 #endif /* _MSC_VER */
 
 #endif /* OBJ_INIT */
+
+
+
+#if defined(_MSC_VER) || defined(M_ix86_darwin)
+#define OBJ_CHAIN_REVERSE_ORDER
+#endif
