@@ -6,7 +6,7 @@
  * Descr.: predicate table management                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred.c,v 1.4 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: pred.c,v 1.10 2007/01/04 10:35:13 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -73,7 +73,7 @@ Init_Pred(void)
  * Called by compiled prolog code, by dynamic predicate support and by     *
  * byte-code support.                                                      *
  *-------------------------------------------------------------------------*/
-PredInf *
+PredInf * FC
 Create_Pred(int func, int arity, int pl_file, int pl_line, int prop,
 	    long *codep)
 {
@@ -107,7 +107,7 @@ Create_Pred(int func, int arity, int pl_file, int pl_line, int prop,
  * LOOKUP_PRED                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-PredInf *
+PredInf * FC
 Lookup_Pred(int func, int arity)
 {
   long key = Functor_Arity(func, arity);
@@ -122,7 +122,7 @@ Lookup_Pred(int func, int arity)
  * DELETE_PRED                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+void FC
 Delete_Pred(int func, int arity)
 {
   long key = Functor_Arity(func, arity);
