@@ -6,7 +6,7 @@
  * Descr.: predicate table management - header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred.h,v 1.6 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: pred.h,v 1.12 2007/01/04 10:35:13 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -69,9 +69,9 @@ extern char *pred_tbl;
 
 void Init_Pred(void);
 
-PredInf *Create_Pred(int func, int arity, int pl_file, int pl_line,
-		     int prop, long *codep) FC;
+PredInf * FC Create_Pred(int func, int arity, int pl_file, int pl_line,
+		     int prop, long *codep);
 
-PredInf *Lookup_Pred(int func, int arity) FC;
+PredInf * FC Lookup_Pred(int func, int arity);
 
-void Delete_Pred(int func, int arity) FC;
+void FC Delete_Pred(int func, int arity);
