@@ -6,7 +6,7 @@
  * Descr.: unification part                                                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: unify.c,v 1.5 2002/03/19 18:24:36 diaz Exp $ */
+/* $Id: unify.c,v 1.11 2007/01/04 10:35:13 diaz Exp $ */
 
 static Bool Check_If_Var_Occurs(WamWord *var_adr, WamWord term_word);
 
@@ -33,7 +33,7 @@ static Bool Check_If_Var_Occurs(WamWord *var_adr, WamWord term_word);
  *    - to define the Unify function (classical unification).              *
  *    - to define the Unify_Occurs_Check function (+ occurs check).        *
  *-------------------------------------------------------------------------*/
-Bool
+Bool FC
 UNIFY_FCT_NAME(WamWord start_u_word, WamWord start_v_word)
 {
   WamWord u_word, u_tag_mask;
