@@ -6,7 +6,7 @@
  * Descr.: WAM instruction implementation - header file                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.            *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_inst.h,v 1.10 2002/06/04 16:51:25 diaz Exp $ */
+/* $Id: wam_inst.h,v 1.18 2007/01/04 10:35:13 diaz Exp $ */
 
 #if 0
 #define GARBAGE_COLLECTOR
@@ -55,11 +55,11 @@
 
 #define ENVIR_STATIC_SIZE          4
 
-#define CPE(e)                     ((WamCont)   (e[-1]))
-#define BCIE(e)                    ((WamWord)   (e[-2]))
-#define EE(e)                      ((WamWord *) (e[-3]))
-#define NBYE(e)                    ((WamWord)   (e[-4]))
-#define Y(e, y)                    ((WamWord)   (e[-5 - (y)]))
+#define CPE(e)                     (*(WamCont *)  &(e[-1]))
+#define BCIE(e)                    (*(WamWord *)  &(e[-2]))
+#define EE(e)                      (*(WamWord **) &(e[-3]))
+#define NBYE(e)                    (*(WamWord *)  &(e[-4]))
+#define Y(e, y)                    (*(WamWord *)  &(e[-5 - (y)]))
 
 #define ENVIR_NAMES                {"CPE", "BCIE", "EE", "NBYE"}
 
@@ -67,10 +67,10 @@
 
 #define ENVIR_STATIC_SIZE          3
 
-#define CPE(e)                     ((WamCont)   (e[-1]))
-#define BCIE(e)                    ((WamWord)   (e[-2]))
-#define EE(e)                      ((WamWord *) (e[-3]))
-#define Y(e, y)                    ((WamWord)   (e[-4 - (y)]))
+#define CPE(e)                     (*(WamCont *)  &(e[-1]))
+#define BCIE(e)                    (*(WamWord *)  &(e[-2]))
+#define EE(e)                      (*(WamWord **) &(e[-3]))
+#define Y(e, y)                    (*(WamWord *)  &(e[-4 - (y)]))
 
 #define ENVIR_NAMES                {"CPE", "BCIE", "EE"}
 
@@ -81,15 +81,15 @@
 
 #define CHOICE_STATIC_SIZE         8
 
-#define ALTB(b)                    ((CodePtr)   (b[-1]))
-#define CPB(b)                     ((WamCont)   (b[-2]))
-#define BCIB(b)                    ((WamWord)   (b[-3]))
-#define EB(b)                      ((WamWord *) (b[-4]))
-#define BB(b)                      ((WamWord *) (b[-5]))
-#define HB(b)                      ((WamWord *) (b[-6]))
-#define TRB(b)                     ((WamWord *) (b[-7]))
-#define CSB(b)                     ((WamWord *) (b[-8]))
-#define AB(b, a)                   ((WamWord)   (b[-9 - (a)]))
+#define ALTB(b)                    (*(CodePtr *)  &(b[-1]))
+#define CPB(b)                     (*(WamCont *)  &(b[-2]))
+#define BCIB(b)                    (*(WamWord *)  &(b[-3]))
+#define EB(b)                      (*(WamWord **) &(b[-4]))
+#define BB(b)                      (*(WamWord **) &(b[-5]))
+#define HB(b)                      (*(WamWord **) &(b[-6]))
+#define TRB(b)                     (*(WamWord **) &(b[-7]))
+#define CSB(b)                     (*(WamWord **) &(b[-8]))
+#define AB(b, a)                   (*(WamWord *)  &(b[-9 - (a)]))
 
 #define CHOICE_NAMES               {"ALTB", "CPB", "BCIB", "EB", "BB", \
                                     "HB", "TRB", "CSB"}
@@ -252,135 +252,135 @@ typedef SwtInf *SwtTbl;
  * Function Prototypes             *
  *---------------------------------*/
 
-WamWord Create_Functor_Arity_Tagged(char *func_str, int arity) FC;
+WamWord FC Create_Functor_Arity_Tagged(char *func_str, int arity);
 
-SwtTbl Create_Swt_Table(int size) FC;
+SwtTbl FC Create_Swt_Table(int size);
 
-void Create_Swt_Atm_Element(SwtTbl t, int size, int atom, CodePtr codep) FC;
+void FC Create_Swt_Atm_Element(SwtTbl t, int size, int atom, CodePtr codep);
 
-void Create_Swt_Stc_Element(SwtTbl t, int size, int func, int arity,
-			    CodePtr codep) FC;
+void FC Create_Swt_Stc_Element(SwtTbl t, int size, int func, int arity,
+			    CodePtr codep);
 
-Bool Get_Atom_Tagged(WamWord w, WamWord start_word) FC;
+Bool FC Get_Atom_Tagged(WamWord w, WamWord start_word);
 
-Bool Get_Atom(int atom, WamWord start_word) FC;
+Bool FC Get_Atom(int atom, WamWord start_word);
 
-Bool Get_Integer_Tagged(WamWord w, WamWord start_word) FC;
+Bool FC Get_Integer_Tagged(WamWord w, WamWord start_word);
 
-Bool Get_Integer(long n, WamWord start_word) FC;
+Bool FC Get_Integer(long n, WamWord start_word);
 
-Bool Get_Float(double n, WamWord start_word) FC;
+Bool FC Get_Float(double n, WamWord start_word);
 
-Bool Get_Nil(WamWord start_word) FC;
+Bool FC Get_Nil(WamWord start_word);
 
-Bool Get_List(WamWord start_word) FC;
+Bool FC Get_List(WamWord start_word);
 
-Bool Get_Structure_Tagged(WamWord w, WamWord start_word) FC;
+Bool FC Get_Structure_Tagged(WamWord w, WamWord start_word);
 
-Bool Get_Structure(int func, int arity, WamWord start_word) FC;
+Bool FC Get_Structure(int func, int arity, WamWord start_word);
 
-WamWord Put_X_Variable(void) FC;
+WamWord FC Put_X_Variable(void);
 
-WamWord Put_Y_Variable(WamWord *y_adr) FC;
+WamWord FC Put_Y_Variable(WamWord *y_adr);
 
-WamWord Put_Unsafe_Value(WamWord start_word) FC;
+WamWord FC Put_Unsafe_Value(WamWord start_word);
 
-WamWord Put_Atom_Tagged(WamWord w) FC;
+WamWord FC Put_Atom_Tagged(WamWord w);
 
-WamWord Put_Atom(int atom) FC;
+WamWord FC Put_Atom(int atom);
 
-WamWord Put_Integer_Tagged(WamWord w) FC;
+WamWord FC Put_Integer_Tagged(WamWord w);
 
-WamWord Put_Integer(long n) FC;
+WamWord FC Put_Integer(long n);
 
-WamWord Put_Float(double n) FC;
+WamWord FC Put_Float(double n);
 
-WamWord Put_Nil(void) FC;
+WamWord FC Put_Nil(void);
 
-WamWord Put_List(void) FC;
+WamWord FC Put_List(void);
 
-WamWord Put_Structure_Tagged(WamWord w) FC;
+WamWord FC Put_Structure_Tagged(WamWord w);
 
-WamWord Put_Structure(int func, int arity) FC;
+WamWord FC Put_Structure(int func, int arity);
 
-WamWord Unify_Variable(void) FC;
+WamWord FC Unify_Variable(void);
 
-void Unify_Void(int n) FC;
+void FC Unify_Void(int n);
 
-Bool Unify_Value(WamWord start_word) FC;
+Bool FC Unify_Value(WamWord start_word);
 
-Bool Unify_Local_Value(WamWord start_word) FC;
+Bool FC Unify_Local_Value(WamWord start_word);
 
-Bool Unify_Atom_Tagged(WamWord w) FC;
+Bool FC Unify_Atom_Tagged(WamWord w);
 
-Bool Unify_Atom(int atom) FC;
+Bool FC Unify_Atom(int atom);
 
-Bool Unify_Integer_Tagged(WamWord w) FC;
+Bool FC Unify_Integer_Tagged(WamWord w);
 
-Bool Unify_Integer(long n) FC;
+Bool FC Unify_Integer(long n);
 
-Bool Unify_Nil(void) FC;
+Bool FC Unify_Nil(void);
 
-Bool Unify_List(void) FC;
+Bool FC Unify_List(void);
 
-Bool Unify_Structure_Tagged(WamWord w) FC;
+Bool FC Unify_Structure_Tagged(WamWord w);
 
-Bool Unify_Structure(int func, int arity) FC;
+Bool FC Unify_Structure(int func, int arity);
 
-WamWord Globalize_If_In_Local(WamWord start_word) FC;
+WamWord FC Globalize_If_In_Local(WamWord start_word);
 
-void Allocate(int n) FC;
+void FC Allocate(int n);
 
-void Deallocate(void) FC;
+void FC Deallocate(void);
 
-CodePtr Switch_On_Term(CodePtr c_var, CodePtr c_atm, CodePtr c_int,
-		       CodePtr c_lst, CodePtr c_stc) FC;
-CodePtr Switch_On_Term_Var_Atm(CodePtr c_var, CodePtr c_atm) FC;
-CodePtr Switch_On_Term_Var_Stc(CodePtr c_var, CodePtr c_stc) FC;
-CodePtr Switch_On_Term_Var_Atm_Lst(CodePtr c_var, CodePtr c_atm,
-				   CodePtr c_lst) FC;
-CodePtr Switch_On_Term_Var_Atm_Stc(CodePtr c_var, CodePtr c_atm,
-				   CodePtr c_stc) FC;
+CodePtr FC Switch_On_Term(CodePtr c_var, CodePtr c_atm, CodePtr c_int,
+		          CodePtr c_lst, CodePtr c_stc);
+CodePtr FC Switch_On_Term_Var_Atm(CodePtr c_var, CodePtr c_atm);
+CodePtr FC Switch_On_Term_Var_Stc(CodePtr c_var, CodePtr c_stc);
+CodePtr FC Switch_On_Term_Var_Atm_Lst(CodePtr c_var, CodePtr c_atm,
+				      CodePtr c_lst);
+CodePtr FC Switch_On_Term_Var_Atm_Stc(CodePtr c_var, CodePtr c_atm,
+				      CodePtr c_stc);
 
-CodePtr Switch_On_Atom(SwtTbl t, int size) FC;
+CodePtr FC Switch_On_Atom(SwtTbl t, int size);
 
-long Switch_On_Integer(void) FC;
+long FC Switch_On_Integer(void);
 
-CodePtr Switch_On_Structure(SwtTbl t, int size) FC;
+CodePtr FC Switch_On_Structure(SwtTbl t, int size);
 
-void Load_Cut_Level(WamWord *word_adr) FC;
+void FC Load_Cut_Level(WamWord *word_adr);
 
-void Cut(WamWord b_word) FC;
+void FC Cut(WamWord b_word);
 
-void Global_Push_Float(double n) FC;
+void FC Global_Push_Float(double n);
 
-double Obtain_Float(WamWord *adr) FC;
+double FC Obtain_Float(WamWord *adr);
 
 
-void Create_Choice_Point(CodePtr codep_alt, int arity) FC;
-void Create_Choice_Point1(CodePtr codep_alt) FC;
-void Create_Choice_Point2(CodePtr codep_alt) FC;
-void Create_Choice_Point3(CodePtr codep_alt) FC;
-void Create_Choice_Point4(CodePtr codep_alt) FC;
+void FC Create_Choice_Point(CodePtr codep_alt, int arity);
+void FC Create_Choice_Point1(CodePtr codep_alt);
+void FC Create_Choice_Point2(CodePtr codep_alt);
+void FC Create_Choice_Point3(CodePtr codep_alt);
+void FC Create_Choice_Point4(CodePtr codep_alt);
 
 
-void Update_Choice_Point(CodePtr codep_alt, int arity) FC;
-void Update_Choice_Point1(CodePtr codep_alt) FC;
-void Update_Choice_Point2(CodePtr codep_alt) FC;
-void Update_Choice_Point3(CodePtr codep_alt) FC;
-void Update_Choice_Point4(CodePtr codep_alt) FC;
+void FC Update_Choice_Point(CodePtr codep_alt, int arity);
+void FC Update_Choice_Point1(CodePtr codep_alt);
+void FC Update_Choice_Point2(CodePtr codep_alt);
+void FC Update_Choice_Point3(CodePtr codep_alt);
+void FC Update_Choice_Point4(CodePtr codep_alt);
 
-void Delete_Choice_Point(int arity) FC;
-void Delete_Choice_Point1(void) FC;
-void Delete_Choice_Point2(void) FC;
-void Delete_Choice_Point3(void) FC;
-void Delete_Choice_Point4(void) FC;
+void FC Delete_Choice_Point(int arity);
+void FC Delete_Choice_Point1(void);
+void FC Delete_Choice_Point2(void);
+void FC Delete_Choice_Point3(void);
+void FC Delete_Choice_Point4(void);
 
-void Untrail(WamWord *low_adr) FC;
+void FC Untrail(WamWord *low_adr);
 
-Bool Unify(WamWord start_u_word, WamWord start_v_word) FC;
+Bool FC Unify(WamWord start_u_word, WamWord start_v_word);
 
-Bool Unify_Occurs_Check(WamWord start_u_word, WamWord start_v_word) FC;
+Bool FC Unify_Occurs_Check(WamWord start_u_word, WamWord start_v_word);
 
 
 #endif /* FRAME_ONLY */
@@ -509,7 +509,8 @@ long chain_len;
       Mem_Word_Cpy(TR, arg, nb);		\
       TR += nb;					\
       Trail_Push(nb);				\
-      Trail_Push(Trail_Tag_Value(TFC, fct));	\
+      Trail_Push(fct);	/*fct adr not aligned*/	\
+      Trail_Push(Trail_Tag_Value(TFC, 0));	\
     }						\
   while (0)
 
