@@ -6,7 +6,7 @@
  * Descr.: Ctrl+C management                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ctrl_c.c,v 1.5 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: ctrl_c.c,v 1.11 2007/01/04 10:35:13 diaz Exp $ */
 
 #include "../EnginePl/gp_config.h"
 
@@ -86,6 +86,14 @@ Wrapper_Handler(int sig)
 
   int from_callback1;
 
+#if defined(__unix__) || defined(__CYGWIN__)
+  sigemptyset(&set);
+  sigaddset(&set, sig);
+  sigprocmask(SIG_UNBLOCK, &set, NULL);
+#elif !defined(WIN32_CONSOLE_CTRL_HANDLER)
+  signal(sig, Wrapper_Handler);
+#endif
+
   if (inside_ctrl_c)
     {
       printf("Already in a Ctrl+C handler - ignored\n");
@@ -94,20 +102,13 @@ Wrapper_Handler(int sig)
     }
   else
     {
-      inside_ctrl_c = 1;
+      inside_ctrl_c = from_callback; /* only 1 if from_callback */
 
       from_callback1 = from_callback;
       from_callback = 0;
       ret_val = (*ctrl_c_handler) (from_callback1);
     }
 
-#if defined(__unix__) || defined(__CYGWIN__)
-  sigemptyset(&set);
-  sigaddset(&set, sig);
-  sigprocmask(SIG_UNBLOCK, &set, NULL);
-#elif !defined(WIN32_CONSOLE_CTRL_HANDLER)
-  signal(sig, Wrapper_Handler);
-#endif
 
   inside_ctrl_c = 0;
 
