@@ -12,7 +12,9 @@
  *---------------------------------*/
 
 #if defined(M_sony_news) || defined(M_ultrix_dec) || defined(M_alpha_osf) ||\
-    defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd)
+    defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd) ||\
+    defined(M_x86_64_linux) || \
+    defined(M_powerpc_bsd) || defined(M_sparc_bsd) || defined(__ELF__)
 
 #   define M_Asm_Symbol1(name)     #name
 #   define M_Asm_Symbol(name)      M_Asm_Symbol1(name)
@@ -54,19 +56,19 @@
                                     asm("jmp $31,($28)," M_Asm_Symbol(lab));\
                                     return;}
 
-#elif defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd)
+#elif defined(M_ix86) && defined(__GNUC__)
 
 #    define M_Direct_Goto(lab)     {asm("jmp " M_Asm_Symbol(lab)); return;}
 
-#elif defined(M_ix86_cygwin)
-
-#    define M_Direct_Goto(lab)     {asm("jmp " M_Asm_Symbol(lab)); return;}
-
-#elif defined(M_ix86_win32)
+#elif defined(_MSC_VER)
 
 #    define M_Direct_Goto(lab)     {_asm {jmp M_Asm_Symbol(lab)}; return;}
 
-#elif defined(M_powerpc_linux)
+#elif defined(M_x86_64_linux)
+
+#    define M_Direct_Goto(lab)     {asm("jmp " M_Asm_Symbol(lab)); return;}
+
+#elif defined(M_powerpc_linux) || defined(M_powerpc_bsd)
 
 #    define M_Direct_Goto(lab)     {asm("b " M_Asm_Symbol(lab)); return;}
 
@@ -85,14 +87,18 @@
 
 #if 0
 
-#if defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd)
+#if defined(M_ix86)
 
 register WamWord *reg_bank asm("ebx");
 
-#elif defined(M_powerpc_linux)
+#elif defined(M_powerpc)
 
 register WamWord *reg_bank asm("r31");
 
+#elif defined(M_x86_64_linux)
+
+register WamWord *reg_bank asm("r12");
+
 #else
 
 WamWord *reg_bank;
