@@ -6,7 +6,7 @@
  * Descr.: translation file for Linux/OSF on alpha                         *
  * Author: Alexander Diemand, Daniel Diaz                                  *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: alpha_any.c,v 1.8 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: alpha_any.c,v 1.14 2007/01/04 10:35:14 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -703,9 +703,7 @@ Here_CP(void)
 {
   Label_Printf("$Lcont%d:", w_label++);
 
-  /* get GP back */
-  /* this has only be introduced to get dynamic object loading work */
-  /* Inst_Printf("ldgp","$gp,0($27)"); */
+  Inst_Printf("ldgp","$gp,0($27)");
 }
 
 
@@ -846,7 +844,8 @@ Move_To_Reg_Y(int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
+Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words, 
+	     char **p_inline)
 {
 }
 
@@ -1284,7 +1283,7 @@ Call_C_Arg_Foreign_D(int offset, int adr_of, int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Invoke(char *fct_name, int nb_args)
+Call_C_Invoke(char *fct_name, int fc, int nb_args, int nb_args_in_words)
 {
   if (!make_inline(fct_name, nb_args))
     {
@@ -1635,16 +1634,7 @@ Data_Start(char *initializer_fct)
   if (initializer_fct == NULL)
     return;
 
-  Label_Printf(".data");
-  Inst_Printf(".align", "3");
-#ifdef M_alpha_linux
-  Inst_Printf(".type", "obj_chain_start,@object");
-  Inst_Printf(".size", "obj_chain_start,32");
-#endif
-  Label_Printf("obj_chain_start:");
-  Inst_Printf(".quad", "%u", OBJ_CHAIN_MAGIC_1);
-  Inst_Printf(".quad", "%u", OBJ_CHAIN_MAGIC_2);
-  Inst_Printf(".quad", "obj_chain_stop");
+  Inst_Printf(".section", ".ctors,\"aw\"");
   Inst_Printf(".quad", "%s", initializer_fct);
 }
 
@@ -1658,19 +1648,4 @@ Data_Start(char *initializer_fct)
 void
 Data_Stop(char *initializer_fct)
 {
-  if (initializer_fct == NULL)
-    return;
-
-#ifdef M_alpha_linux
-  Label_Printf(".section\t.sdata,\"aw\"");
-#else
-  Label_Printf(".data");
-#endif
-  Inst_Printf(".align", "3");
-#ifdef M_alpha_linux
-  Inst_Printf(".type", "obj_chain_stop,@object");
-  Inst_Printf(".size", "obj_chain_stop,8");
-#endif
-  Label_Printf("obj_chain_stop:");
-  Inst_Printf(".quad", "obj_chain_start");
 }
