@@ -6,7 +6,7 @@
  * Descr.: code generation                                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ma2asm.c,v 1.8 2002/03/19 18:24:37 diaz Exp $ */
+/* $Id: ma2asm.c,v 1.15 2007/01/04 10:35:14 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,8 +45,6 @@
 
 #define DEFAULT_OUTPUT_SUFFIX      ASM_SUFFIX
 
-#define LONG_STACK_SIZE            8192
-
 #define MASK_LONG_GLOBAL           1
 #define MASK_LONG_INITIALIZED      2
 
@@ -57,17 +55,21 @@
  * Type Definitions                *
  *---------------------------------*/
 
-typedef struct inflong *InfLongP;
-
-typedef struct inflong
+typedef struct
 {
-  char *name;
   int global;
   VType vtype;			/* NONE, INITIAL_VALUE, ARRAY_SZIE */
   long value;
-  InfLongP next;
 }
-InfLong;
+LongInf;
+
+
+typedef struct
+{
+  int prolog;
+  int global;
+}
+CodeInf;
 
 
 
@@ -79,6 +81,7 @@ InfLong;
 char *file_name_in;
 char *file_name_out;
 int inline_asm;
+int ignore_fc;
 int comment;
 
 FILE *file_out;
@@ -86,10 +89,8 @@ FILE *file_out;
 int work_label = 0;
 
 BTString bt_string;
-
-InfLong dummy_long_start;
-InfLong *long_end = &dummy_long_start;
-int nb_long;
+BTString bt_code;
+BTString bt_long;
 
 char *initializer_fct = NULL;
 
@@ -100,6 +101,8 @@ char *initializer_fct = NULL;
  * Function Prototypes             *
  *---------------------------------*/
 
+void Invoke_Dico_Long(int unused_no, char *name, LongInf *p);
+
 void Init_Inline_Data(void);
 
 char **Find_Inline_Data(char *fct_name);
@@ -142,7 +145,6 @@ void Display_Help(void);
 int
 main(int argc, char *argv[])
 {
-  InfLong *p;
   int n;
 
   Parse_Arguments(argc, argv);
@@ -158,6 +160,8 @@ main(int argc, char *argv[])
   Init_Inline_Data();
 
   BT_String_Init(&bt_string);
+  BT_String_Init(&bt_code);	/* only fill if Pre_Pass is asked */
+  BT_String_Init(&bt_long);
   Asm_Start();
 
   if (!Parse_Ma_File(file_name_in, comment))
@@ -172,20 +176,16 @@ main(int argc, char *argv[])
   if (n)
     {
       Dico_String_Start(n);
-
       BT_String_List(&bt_string, Dico_String);
-
       Dico_String_Stop(n);
     }
 
-  if (nb_long)
+  n = bt_long.nb_elem;
+  if (n)
     {
-      Dico_Long_Start(nb_long);
-
-      for (p = dummy_long_start.next; p; p = p->next)
-	Dico_Long(p->name, p->global, p->vtype, p->value);
-
-      Dico_Long_Stop(nb_long);
+      Dico_Long_Start(n);
+      BT_String_List(&bt_long, Invoke_Dico_Long);
+      Dico_Long_Stop(n);
     }
 
   Data_Stop(initializer_fct);
@@ -201,14 +201,27 @@ main(int argc, char *argv[])
 
 
 
+/*-------------------------------------------------------------------------*
+ * INVOKE_DICO_LONG                                                        *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Invoke_Dico_Long(int unused_no, char *name, LongInf *p)
+{
+  Dico_Long(name, p->global, p->vtype, p->value);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * DECLARE_INITIALIZER                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
 Declare_Initializer(char *init_fct)
-{
-  initializer_fct = strdup(init_fct);
+{				/* init_fct: strdup done by the parser */
+  initializer_fct = init_fct;
 }
 
 
@@ -219,7 +232,7 @@ Declare_Initializer(char *init_fct)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C(char *fct_name, int fc, int nb_args, ArgInf arg[])
+Call_C(char *fct_name, int fc, int nb_args, int nb_args_in_words, ArgInf arg[])
 {
   unsigned i;			/* unsigned is important for the loop */
   int inc;
@@ -236,7 +249,10 @@ Call_C(char *fct_name, int fc, int nb_args, ArgInf arg[])
   }
 #endif
 
-  Call_C_Start(fct_name, fc, nb_args, p_inline);
+  if (ignore_fc)
+    fc = 0;
+
+  Call_C_Start(fct_name, fc, nb_args, nb_args_in_words, p_inline);
 
   if (!call_c_reverse_args)
     i = 0, inc = 1;
@@ -296,7 +312,7 @@ Call_C(char *fct_name, int fc, int nb_args, ArgInf arg[])
       Emit_Inline_Data(p_inline);
     }
   else
-    Call_C_Invoke(fct_name, nb_args);
+    Call_C_Invoke(fct_name, fc, nb_args, nb_args_in_words);
   
   if (p_inline && comment)
     Label_Printf("\t\t%s code after inlining (Call_C_Stop)", comment_prefix);
@@ -488,27 +504,67 @@ Switch_Cmp_Int(SwtInf *c1, SwtInf *c2)
 void
 Decl_Long(char *name, int global, VType vtype, long value)
 {
-  InfLong *p;
+  LongInf *p;
 
-  nb_long++;
+		/* name: strdup done by the parser */
+  p = (LongInf *) BT_String_Add(&bt_long, name)->info;
 
-  p = (InfLong *) malloc(sizeof(InfLong));
-  if (p == NULL)
-    {
-      fprintf(stderr, "Cannot allocate memory for long %s\n", name);
-      exit(1);
-    }
-
-
-  p->name = name;		/* strdup done by the parser */
   p->global = global;
-
   p->vtype = vtype;
   p->value = value;
-  p->next = NULL;
+}
 
-  long_end->next = p;
-  long_end = p;
+
+
+
+/*-------------------------------------------------------------------------*
+ * DECL_CODE                                                               *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void				/* called by Pre_Pass */
+Decl_Code(char *name, int prolog, int global)
+{
+  CodeInf *p;
+
+		/* name: strdup done by the parser */
+  p = (CodeInf *) BT_String_Add(&bt_code, name)->info;
+
+  p->prolog = prolog;
+  p->global = global;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * IS_CODE_DEFINED                                                         *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Is_Code_Defined(char *name)
+{
+  return (BT_String_Lookup(&bt_code, name) != NULL);
+}
+
+
+/*-------------------------------------------------------------------------*
+ * GET_LONG_INFOS                                                          *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Get_Long_Infos(char *name, int *global, VType *vtype, int *value)
+{
+  BTNode *b = BT_String_Lookup(&bt_long, name);
+  LongInf *p;
+
+  if (b == NULL)
+    return 0;
+
+  p = (LongInf *) b->info;
+  *global = p->global;
+  *vtype = p->vtype;
+  *value = p->value;
+  return 1;
 }
 
 
@@ -625,6 +681,7 @@ Parse_Arguments(int argc, char *argv[])
 
   file_name_in = file_name_out = NULL;
   inline_asm = 0;
+  ignore_fc = 0;
   comment = 0;
 
   for (i = 1; i < argc; i++)
@@ -656,6 +713,12 @@ Parse_Arguments(int argc, char *argv[])
 	      continue;
 	    }
 
+	  if (Check_Arg(i, "--ignore-fast"))
+	    {
+	      ignore_fc = 1;
+	      continue;
+	    }
+
 	  if (Check_Arg(i, "--comment"))
 	    {
 	      comment = 1;
@@ -724,6 +787,7 @@ Display_Help(void)
   L("  -o FILE, --output FILE      set output file name");
   L("  --inline-asm                inline some C calls as asm instructions");
   L("  --full-inline-asm           inline most C calls as asm instructions");
+  L("  --ignore-fast               ignore fast call (FC) declarations");
   L("  --comment                   include comments in the output file");
   L("  -h, --help                  print this help and exit");
   L("  --version                   print version number and exit");
