@@ -6,7 +6,7 @@
  * Descr.: translation file                                                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ma2asm_inst.c,v 1.8 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: ma2asm_inst.c,v 1.16 2007/01/04 10:35:14 diaz Exp $ */
 
 #define MAPPER_FILE
 
@@ -42,18 +42,14 @@
 
 #if 0				/* to force the inclusion of a mapper */
 
-#define M_powerpc_darwin
-#include "powerpc_any.c"
+#define M_ix86_darwin
+#include "ix86_any.c"
 
 #else
 
 	  /* include machine-dependent mapper file */
 
-#if defined(M_ix86_win32)
-
-#include "ix86_win32.c"
-
-#elif defined(M_ix86)
+#if defined(M_ix86)
 
 #include "ix86_any.c"
 
@@ -73,7 +69,12 @@
 
 #include "powerpc_any.c"
 
+#elif defined(M_x86_64)
+
+#include "x86_64_any.c"
+
 #endif
 
 #endif
 
+
