@@ -6,7 +6,7 @@
  * Descr.: code generation - header file                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ma_protos.h,v 1.9 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: ma_protos.h,v 1.16 2007/01/04 10:35:14 diaz Exp $ */
 
 #if 0
 #define CHECK_PRINTF_ARGS
@@ -81,15 +81,24 @@ extern int reload_e;
 
 void Declare_Initializer(char *initializer_fct);
 
-void Call_C(char *fct_name, int fc, int nb_arg, ArgInf arg[]);
+void Call_C(char *fct_name, int fc, int nb_args, int nb_args_in_words, ArgInf arg[]);
 
 void Switch_Ret(int nb_swt, SwtInf swt[]);
 
+void Decl_Code(char *name, int prolog, int global);
+
 void Decl_Long(char *name, int global, VType vtype, long value);
 
 
+
+
+
 	  /* defined in ma2asm.c - used by mappers */
 
+int Is_Code_Defined(char *name);
+
+int Get_Long_Infos(char *name, int *global, VType *vtype, int *value);
+
 void Label_Printf(char *label, ...) GCCPRINTF(1);
 
 void Inst_Printf(char *op, char *operands, ...) GCCPRINTF(2);
@@ -139,7 +148,8 @@ void Move_To_Reg_X(int index);
 
 void Move_To_Reg_Y(int index);
 
-void Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline);
+void Call_C_Start(char *fct_name, int fc, int nb_args, 
+		  int nb_args_in_words, char **p_inline);
 
 int Call_C_Arg_Int(int offset, long int_val);
 
@@ -157,7 +167,7 @@ int Call_C_Arg_Foreign_L(int offset, int adr_of, int index);
 
 int Call_C_Arg_Foreign_D(int offset, int adr_of, int index);
 
-void Call_C_Invoke(char *fct_name, int nb_args);
+void Call_C_Invoke(char *fct_name, int fc, int nb_args, int nb_args_in_words);
 
 void Call_C_Stop(char *fct_name, int nb_args, char **p_inline);
 
