@@ -6,7 +6,7 @@
  * Descr.: translation file for IRIX on MIPS                               *
  * Author: Alexander Diemand, Daniel Diaz                                  *
  *                                                                         *
- * Copyright (C) 1999 Daniel Diaz                                          *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: mips_irix.c,v 1.6 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: mips_irix.c,v 1.10 2007/01/04 10:35:14 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -484,7 +484,8 @@ Move_To_Reg_Y(int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
+Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words, 
+	     char **p_inline)
 {
 }
 
@@ -977,7 +978,7 @@ Call_C_Arg_Foreign_D(int offset, int adr_of, int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Invoke(char *fct_name, int nb_args)
+Call_C_Invoke(char *fct_name, int fc, int nb_args, int nb_args_in_words)
 {
 /*  if (!make_inline (fct_name, nb_args)) { */
   Inst_Printf("sd", "$gp,%d($sp)", MAX_C_ARGS_IN_C_CODE * 8 + 8);
@@ -1300,21 +1301,8 @@ Data_Start(char *initializer_fct)
   if (initializer_fct == NULL)
     return;
 
-  Label_Printf("\t.section\t.rodata");
-  Label_Printf("\t.data");
-  Inst_Printf(".align", "3");
-  Inst_Printf(".size", "obj_chain_start,16");
-  Label_Printf("obj_chain_start:");
-
-  Inst_Printf(".word", "%d", OBJ_CHAIN_MAGIC_1);
-  Inst_Printf(".word", "%d", OBJ_CHAIN_MAGIC_2);
-  Inst_Printf(".word", "obj_chain_stop");
+  Inst_Printf(".section", ".ctors,\"aw\",@progbits");
   Inst_Printf(".word", "%s", initializer_fct);
-
-  Inst_Printf(".align", "2");
-  Inst_Printf(".size", "obj_chain_stop,4");
-  Label_Printf("obj_chain_stop:");
-  Inst_Printf(".word", "obj_chain_start");
 }
 
 
@@ -1327,6 +1315,4 @@ Data_Start(char *initializer_fct)
 void
 Data_Stop(char *initializer_fct)
 {
-  if (initializer_fct == NULL)
-    return;
 }
