@@ -6,7 +6,7 @@
  * Descr.: translation file for Linux/Darwin (MacOsX) on PowerPC           *
  * Author: Daniel Diaz and Lindsey Spratt                                  *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: powerpc_any.c,v 1.2 2002/05/15 10:02:41 diaz Exp $ */
+/* $Id: powerpc_any.c,v 1.9 2007/01/04 10:35:14 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -42,7 +42,7 @@
 #define MAX_DOUBLES_IN_PRED        2048
 
 
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
 
 #define UN
 #define R(reg)                     #reg
@@ -161,6 +161,9 @@ Asm_Start(void)
 void
 Asm_Stop(void)
 {
+#ifdef __ELF__
+  Inst_Printf(".section", ".note.GNU-stack,\"\",@progbits");
+#endif
 }
 
 
@@ -186,7 +189,7 @@ Code_Start(char *label, int prolog, int global)
 
   Label_Printf("");
   Inst_Printf(".align", "2");
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
   Inst_Printf(".type", "%s,@function", label);
 #endif
 
@@ -423,7 +426,8 @@ Move_To_Reg_Y(int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
+Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words, 
+	     char **p_inline)
 {
   dbl_reg_no = 0;
 }
@@ -431,7 +435,7 @@ Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
 
 
 
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
 
 #define STACK_OFFSET(offset)   offset * 4 - 24
 #define DBL_RET_WORDS          0
@@ -462,7 +466,7 @@ Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
 }
 
 
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
 
 #define AFTER_ARG_DBL						\
 }
@@ -649,7 +653,7 @@ Call_C_Arg_Foreign_D(int offset, int adr_of, int index)
   BEFORE_ARG;
 
   Inst_Printf("addis", "%s,0," HI_UN(foreign_double+%d), r,
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
 	      index * 4
 #else
 	      index * 8
@@ -682,7 +686,7 @@ Call_C_Arg_Foreign_D(int offset, int adr_of, int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Invoke(char *fct_name, int nb_args)
+Call_C_Invoke(char *fct_name, int fc, int nb_args, int nb_args_in_words)
 {
 #if 0	/* only useful to call varargs functions - not the case here */
   if (dbl_reg_no == 0)
@@ -882,7 +886,7 @@ C_Ret(void)
 void
 Dico_String_Start(int nb_consts)
 {
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
   Label_Printf(".section\t.rodata");
 #else
   Label_Printf(".cstring");
@@ -900,7 +904,7 @@ void
 Dico_String(int str_no, char *asciiz)
 {
   Label_Printf("%s%d:", STRING_PREFIX, str_no);
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
   Inst_Printf(".string", "%s", asciiz);
 #else
   Inst_Printf(".asciz", "%s", asciiz);
@@ -948,7 +952,7 @@ Dico_Long(char *name, int global, VType vtype, long value)
     case NONE:
       value = 1;		/* then in case ARRAY_SIZE */
     case ARRAY_SIZE:
-#ifdef M_powerpc_linux
+#if defined(M_powerpc_linux) || defined(M_powerpc_bsd)
       if (!global)
 	Inst_Printf(".local", UN "%s", name);
       Inst_Printf(".comm", UN "%s,%ld,4", name, value * 4);
@@ -994,13 +998,16 @@ Data_Start(char *initializer_fct)
   if (initializer_fct == NULL)
     return;
 
-  Label_Printf(".data");
-  Label_Printf(UN "obj_chain_start:");
-
-  Inst_Printf(".long", "%d", OBJ_CHAIN_MAGIC_1);
-  Inst_Printf(".long", "%d", OBJ_CHAIN_MAGIC_2);
-  Inst_Printf(".long", UN "obj_chain_stop");
+#ifdef M_powerpc_linux
+  Inst_Printf(".section", ".ctors,\"aw\",@progbits");
+  Inst_Printf(".align", "2");
   Inst_Printf(".long", UN "%s", initializer_fct);
+#else
+  Label_Printf(".data");
+  Label_Printf(".mod_init_func");
+  Inst_Printf(".align", "2");
+  Inst_Printf(".long", UN "%s", initializer_fct);
+#endif
 }
 
 
@@ -1016,8 +1023,10 @@ Data_Stop(char *initializer_fct)
   if (initializer_fct == NULL)
     return;
 
+#if 0
   Label_Printf(".data");
   Label_Printf(UN "obj_chain_stop:");
 
   Inst_Printf(".long", UN "obj_chain_start");
+#endif
 }
