@@ -6,7 +6,7 @@
  * Descr.: translation file for SunOs/Solaris on sparc                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: sparc_any.c,v 1.9 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: sparc_any.c,v 1.17 2007/01/04 10:35:14 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -67,14 +67,17 @@ char asm_reg_cp[16];
 int w_label = 0;
 
 char *delay_op;
-char delay_operands[64];
+char delay_operands[1024];
 
 
 
 
 	  /* variables for ma_parser.c / ma2asm.c */
-
-char *comment_prefix = "#";
+#if 0
+char *comment_prefix = "#";  /* does not work on solaris 9 */
+#else
+char *comment_prefix = "!";
+#endif
 char *local_symb_prefix = "L";
 int strings_need_null = 1;
 int call_c_reverse_args = 0;
@@ -171,7 +174,7 @@ Code_Start(char *label, int prolog, int global)
 {
   Label_Printf("");
   Inst_Printf(".align", "4");
-#ifdef M_sparc_solaris
+#if defined(M_sparc_solaris) || defined(M_sparc_bsd)
   Inst_Printf(".type", UN "%s,#function", label);
 #endif
   Inst_Printf(".proc", "020");
@@ -249,8 +252,9 @@ Pl_Jump(char *label)
 void
 Prep_CP(void)
 {
-  fprintf(stderr, "PREP_CP not implemented\n");
-  exit(1);
+  Inst_Printf("sethi", "%%hi(.Lcont%d-8),%%g1", w_label);
+  Inst_Printf("or", "%%g1,%%lo(.Lcont%d-8),%%g1", w_label);
+  Inst_Printf("st", "%%g1,%s", asm_reg_cp);
 }
 
 
@@ -263,8 +267,7 @@ Prep_CP(void)
 void
 Here_CP(void)
 {
-  fprintf(stderr, "HERE_CP not implemented\n");
-  exit(1);
+  Label_Printf(".Lcont%d:", w_label++);
 }
 
 
@@ -399,7 +402,8 @@ Move_To_Reg_Y(int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
+Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words, 
+	     char **p_inline)
 {
   delay_op = NULL;
 }
@@ -635,7 +639,7 @@ Call_C_Arg_Foreign_D(int offset, int adr_of, int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Invoke(char *fct_name, int nb_args)
+Call_C_Invoke(char *fct_name, int fc, int nb_args, int nb_args_in_words)
 {
   Inst_Printf("call", UN "%s", fct_name);
   if (delay_op)
@@ -839,6 +843,7 @@ C_Ret(void)
 void
 Dico_String_Start(int nb_consts)
 {
+  Inst_Printf(".section", ".rodata");
 }
 
 
@@ -853,7 +858,7 @@ Dico_String(int str_no, char *asciiz)
 {
   Inst_Printf(".align", "8");
   Label_Printf("%s%d:", STRING_PREFIX, str_no);
-  Inst_Printf(".ascii", "%s", asciiz);
+  Inst_Printf(".asciz", "%s", asciiz);
 }
 
 
@@ -915,7 +920,7 @@ Dico_Long(char *name, int global, VType vtype, long value)
       break;
 
     case INITIAL_VALUE:
-#ifdef M_sparc_solaris
+#if defined(M_sparc_solaris) || defined(M_sparc_bsd)
       Inst_Printf(".type", UN "%s,#object", name);
       Inst_Printf(".size", UN "%s,4", name);
 #endif
@@ -956,14 +961,9 @@ Data_Start(char *initializer_fct)
   if (initializer_fct == NULL)
     return;
 
-  Label_Printf(".data");
-  Label_Printf("\t.align 4");
-  Label_Printf(UN "obj_chain_start:");
-
-  Inst_Printf(".word", "%d", OBJ_CHAIN_MAGIC_1);
-  Inst_Printf(".word", "%d", OBJ_CHAIN_MAGIC_2);
-  Inst_Printf(".word", UN "obj_chain_stop");
-  Inst_Printf(".word", UN "%s", initializer_fct);
+  Inst_Printf(".section", ".ctors,#alloc,#write");
+  Inst_Printf(".align", "4");
+  Inst_Printf(".long", UN "%s", initializer_fct);
 }
 
 
@@ -976,14 +976,6 @@ Data_Start(char *initializer_fct)
 void
 Data_Stop(char *initializer_fct)
 {
-  if (initializer_fct == NULL)
-    return;
-
-  Label_Printf(".data");
-  Label_Printf("\t.align 4");
-  Label_Printf(UN "obj_chain_stop:");
-
-  Inst_Printf(".word", UN "obj_chain_start");
 }
 
 
