@@ -6,7 +6,7 @@
  * Descr.: pass 2: internal format transformation                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: internal.pl,v 1.9 2002/04/19 15:43:36 diaz Exp $ */
+/* $Id: internal.pl,v 1.14 2007/01/04 10:34:06 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * predicate internal format: (I(t)=internal format of t)                  *
