@@ -6,7 +6,7 @@
  * Descr.: compiler main (shell) program                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_comp.c,v 1.11 2002/05/10 16:35:26 diaz Exp $ */
+/* $Id: top_comp.c,v 1.17 2007/01/04 10:35:15 diaz Exp $ */
 
 #include "../EnginePl/gp_config.h"
 
@@ -35,7 +35,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
 #include <windows.h>
 #include <process.h>
 #include <fcntl.h>
@@ -71,8 +71,6 @@
 
 #define TEMP_FILE_PREFIX           GPLC
 
-#define OBJ_FILE_OBJ_BEGIN         "obj_begin"
-#define OBJ_FILE_OBJ_END           "obj_end"
 #define OBJ_FILE_ALL_PL_BIPS       "all_pl_bips"
 #define OBJ_FILE_ALL_FD_BIPS       "all_fd_bips"
 #define OBJ_FILE_TOP_LEVEL         "top_level"
@@ -282,7 +280,7 @@ main(int argc, char *argv[])
 {
   char **pdev;
 
-#ifdef M_ix86_win32
+#ifdef _WIN32
   setbuf(stdout, NULL);
   setbuf(stderr, NULL);
 #endif
@@ -631,6 +629,7 @@ Link_Cmd(void)
   static char file_out[MAXPATHLEN];
   static char buff[CMD_LINE_LENGTH];
   FileInf *f;
+  int has_gui_console = 0;
 
 
   if (file_name_out == NULL)
@@ -642,13 +641,31 @@ Link_Cmd(void)
   Create_Output_File_Name(f, file_out);
   file_name_out = file_out;
 
+  /* with MSVC: if at run-time we don't find cl.exe we use link.exe
+   * it is a workaround for users who have installed a binary version
+   * (from a setup.exe) compiled with cl.exe but who don't have cl.exe
+   */
+#ifdef _MSC_VER
+  {
+    char *dont_care;
+
+    if (SearchPath(NULL, cmd_link.exe_name, ".exe", CMD_LINE_LENGTH, buff, &dont_care) && getenv("USE_LINKER") == NULL)
+#endif
+
   sprintf(buff, "%s%s%s%s ", cmd_link.exe_name, cmd_link.opt,
 	  cmd_link.out_opt, file_name_out);
 
-  Find_File(OBJ_FILE_OBJ_BEGIN, OBJ_SUFFIX, buff + strlen(buff));
-  strcat(buff, " ");
+#ifdef _MSC_VER
+    else
+      {
+	if (verbose)
+	  printf("%s.exe not found ! we use link.exe\n", cmd_link.exe_name);
+	sprintf(buff, "link /nologo /stack:8000000 /out:%s ", file_name_out);
+      }
+  }
+#endif
 
-				/* f->work_name1 is OK for LINK_OPTION */
+  /* f->work_name1 is OK for LINK_OPTION */
   for (f = file_lopt; f->name; f++)
     sprintf(buff + strlen(buff), "%s ", f->work_name1);
 
@@ -695,9 +712,6 @@ Link_Cmd(void)
       strcat(buff, " ");
     }
 
-  Find_File(OBJ_FILE_OBJ_END, OBJ_SUFFIX, buff + strlen(buff));
-  strcat(buff, " ");
-
   Find_File(LIB_ENGINE_PL, "", buff + strlen(buff));
   strcat(buff, " ");
 
@@ -708,18 +722,21 @@ Link_Cmd(void)
 
   strcat(buff, LDLIBS " ");
 
-  if (!no_pl_lib)
-    {
-      if (gui_console)
-	{    /* modify Linedit/Makefile.in to follow this list of ld objects */
-	  Find_File("w32gc_interf.obj", "", buff + strlen(buff));
-	}
-#ifdef M_ix86_win32
-      else
-	strcat(buff, "/link /subsystem:console ");
-#endif
+  if (!no_pl_lib && gui_console) 
+    {    /* modify Linedit/Makefile.in to follow this list of ld objects */
+      Find_File("w32gc_interf", OBJ_SUFFIX, buff + strlen(buff));
+      strcat(buff, " ");
+      has_gui_console = 1;
     }
 
+#ifdef _MSC_VER
+  if (*buff != 'l' && *buff != 'L') /* it is not link.exe ! */
+    strcat(buff, "/link ");
+  strcat(buff, "/ignore:4089 ");
+  if (!has_gui_console)
+    strcat(buff, "/subsystem:console ");
+#endif
+
   Exec_One_Cmd(buff, no_decode_hex);
 
   if (strip && *EXE_FILE_STRIP != ':' && *EXE_FILE_STRIP != '\0')
@@ -1269,7 +1286,7 @@ Parse_Arguments(int argc, char *argv[])
 
       for(q = f->suffix; q >= f->name; q--)
 	if (*q == '/'
-#ifdef M_ix86_win32
+#ifdef _WIN32
 	    || *q == '\\'
 #endif
 	    )
@@ -1399,7 +1416,7 @@ Display_Help(void)
   L("  --no-debugger               do not link the Prolog/WAM debugger");
   L("  --min-pl-bips               link only used Prolog built-in predicates");
   L("  --min-fd-bips               link only used FD solver built-in predicates");
-  L("  --min-bips                  same as: --no-top-level --min-pl-bips --min-fd-bips");
+  L("  --min-bips                  same as: --no-top-level --min-pl-bips --min-fd-bips --no-debugger");
   L("  --min-size                  same as: --min-bips --strip");
   L("  --no-pl-lib                 do not look for the Prolog and FD libraries (maintenance only)");
   L("  --no-fd-lib                 do not look for the FD library (maintenance only)");
