@@ -6,7 +6,7 @@
  * Descr.: line editor <--> GUICons interface                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2002 Daniel Diaz                                     *
+ * Copyright (C) 1999-2007 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -19,10 +19,10 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
  *-------------------------------------------------------------------------*/
 
-/* $Id: w32gc_interf.c,v 1.7 2002/03/19 18:24:38 diaz Exp $ */
+/* $Id: w32gc_interf.c,v 1.12 2007/01/04 10:35:15 diaz Exp $ */
 
 #include <stdio.h>
 #include <windows.h>
@@ -88,21 +88,20 @@ void (*le_hook_exit_process) ();
 static FARPROC Find_Fct(HANDLE h, char *name);
 
 
-typedef
-__declspec(dllimport)
-     int (*Fct) ();
+typedef __declspec(dllimport) int (*Fct) ();
 
 
 /*-------------------------------------------------------------------------*
  * START_GUI                                                               *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-     static void Start_GUI(void)
+static void
+Start_GUI(void)
 {
   Fct W32GC_Start_Window;
   HANDLE h;
 
-#if 1
+#if 1 /* O to force console mode */
   h = LoadLibrary(DLL_W32GUICONS);
 #else
   h = NULL;
@@ -116,7 +115,6 @@ __declspec(dllimport)
       le_hook_start = NULL;
       return;
     }
-
   le_hook_put_char = (void (*)()) Find_Fct(h, "_W32GC_Put_Char");
   le_hook_get_char0 = (int (*)()) Find_Fct(h, "_W32GC_Get_Char0");
   le_hook_kbd_is_not_empty =
@@ -158,12 +156,14 @@ __declspec(dllimport)
 static FARPROC
 Find_Fct(HANDLE h, char *name)
 {
-  FARPROC p = GetProcAddress(h, name + 1);
+  FARPROC p;			/* init here to avoid lcc (buggy) warning */
+
+  p = GetProcAddress(h, name + 1);
 
   if (p)
     return p;
 
-  /* use necessary for MSVC++ but useful for lcc for instance */
+  /* useless for MSVC++ but useful for lcc for instance */
 
   p = GetProcAddress(h, name);
 
